/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <cmath>
#include <o2scl/test_mgr.h>
#include <o2scl/mm_funct.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/gsl_mroot_hybrids.h>
#include <o2scl/cern_mroot.h>
#include <o2scl/gsl_rnga.h>

int gfn(size_t nv, const o2scl::ovector_view &x, 
	o2scl::ovector_view &y, double &pa) {
  y[0]=sin(x[1]-pa);
  y[1]=sin(x[0]-pa*2.0)+exp(x[0]-pa);
  return 0;
}

using namespace std;
using namespace o2scl;

int tti(double x) {
  return ((int)(-log10(fabs(x))));
}

int main(void) {
  ovector x(2), y(2);
  gsl_rnga gr;
  double pa;
  test_mgr t;
  size_t tmp;
  const size_t N=5000;
  double t1;

  cout.setf(ios::scientific);

  t.set_output_level(1);

  const int max_index=15;
  double time[max_index];
  size_t n[max_index];
  double time2[max_index];
  size_t n2[max_index];
  for(int i=0;i<max_index;i++) {
    n[i]=0;
    n2[i]=0;
  }
  double acc;
  size_t NX=10;

  mm_funct_fptr<double,ovector_view> mff(gfn);
  gsl_mroot_hybrids<double> cr1;
  cern_mroot<double> cr2;
      
  for(size_t j=0;j<60;j++) {
    cout << "." << flush;
	
    cr1.tolf=pow(10.0,-4-((double)j)/9.0);
	
    acc=0.0;
    tmp=clock();
    for(size_t ix=0;ix<NX;ix++) {
      pa=gr.random()/2.0;
      for(size_t k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	int ret=cr1.msolve(2,x,pa,mff);
	err_assert(ret);
      }
      gfn(2,x,y,pa);
      acc+=fabs(y[0])+fabs(y[1]);
    }
    acc/=((double)NX);
    t1=(clock()-tmp)/10000;
    int ix=tti(acc);
    if (ix>=0 && ix<max_index) {
      n[ix]++;
      time[ix]+=t1;
    }
	
  }
  cout << endl;

  for(size_t j=0;j<60;j++) {
    cout << "." << flush;
    
    cr2.tolx=pow(10.0,-2-((double)j)/18.0);
    cr2.tolf=pow(10.0,-2-((double)j)/18.0);
	
    acc=0.0;
    tmp=clock();

    for(size_t ix=0;ix<NX;ix++) {
      pa=gr.random()/2.0;
      for(size_t k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	int ret=cr2.msolve(2,x,pa,mff);
	err_assert(ret);
      }
      gfn(2,x,y,pa);
      acc+=fabs(y[0])+fabs(y[1]);
    }
    acc/=((double)NX);
    t1=(clock()-tmp)/10000;
    int ix=tti(acc);
    if (ix>=0 && ix<max_index) {
      n2[ix]++;
      time2[ix]+=t1;
    }
	
  }
  cout << endl;

  cout << "Accuracy   gsl_mroot_hybrids   cern_mroot" << endl;
  cout << "--------------------------------------------------" << endl;
  for(int j=0;j<max_index;j++) {
    if (n[j]>0 && n2[j]>0) {
      cout.width(2);
      cout << j << "         ";
      cout << time[j]/n[j] << "        " << time2[j]/n2[j] << " "
	   << n[j] << " " << n2[j] << endl;
      //} else {
      //cout << n[j] << " " << n2[j] << endl;
    }
    
  }

  t.report();
  return 0;
}
