/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_TPTR_SCHEDULE_H
#define O2SCL_TPTR_SCHEDULE_H

#include <iostream>
#include <o2scl/collection.h>
#include <o2scl/ovector_tlate.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Simulated annealing temperature schedule base

      The schedules are designed to be used in the following way

      \code
      for(temper=tp->start(fmin,nvar,pb,NULL);
      tp->done(fmin,nvar,pb,NULL)==false;
      temper=tp->next(fmin,nvar,pb,NULL)) {
      }
      \endcode

  */
  template<class vec_t=ovector_view> class tptr_schedule {

    public:
    
    virtual ~tptr_schedule() {}
    
    /// Return the first temperature
    virtual double start(double min, int nv, const vec_t &best, void *vp) 
    { return 0.0; }
    
    /// Return the next temperature
    virtual double next(double min, int nv, const vec_t &best, void *vp) 
    { return 0.0; }
    
    /// Return true if the last step made the temperature too small
    virtual bool done(double min, int nv, const vec_t &best, void *vp) 
    { return true; }
    
    /// Return string denoting type ("tptr_schedule")
    virtual const char *type() { return "tptr_schedule"; }
  };
  
#ifndef DOXYGENP
}
#endif

#endif

