/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/binary_file.h>

using namespace std;
using namespace o2scl;

// ------------------------------------------------------------
// binary_out_file functions
// ------------------------------------------------------------

binary_out_file::binary_out_file(string file_name) {

  int len=file_name.length();
  if (file_name[len-1]=='z' && file_name[len-2]=='g' &&
      file_name[len-3]=='.') {
    
    user_filename=file_name.substr(0,len-3);
    
    string stemplat;
    int ir=o2scl_make_tfile(stemplat);
    outs.open(stemplat.c_str(),ios::binary);

    temp_filename=stemplat;

    compressed=true;
    gzip=true;
  } else if (file_name[len-1]=='2' && file_name[len-2]=='z' &&
	     file_name[len-3]=='b' && file_name[len-4]=='.') {
    user_filename=file_name.substr(0,len-4);

    string stemplat;
    int ir=o2scl_make_tfile(stemplat);
    outs.open(stemplat.c_str(),ios::binary);

    temp_filename=stemplat;

    compressed=true;
    gzip=false;
  } else {
    outs.open(file_name.c_str(),ios::binary);
    
    compressed=false;
  }
}

binary_out_file::~binary_out_file() {
  
  outs.close();

  if (compressed) {
    string cmd;
    int ret;
    
    cmd=((string)"mv ")+temp_filename+" "+user_filename;
    ret=system(cmd.c_str());
    if (ret!=0) {
      set_err("Creating file failed. File closed anyway.",-1);
      return;
    }
    
    if (gzip) cmd=((string)"gzip -f ")+user_filename;
    else cmd=((string)"bzip2 -f ")+user_filename;
    ret=system(cmd.c_str());
    if (ret!=0) {
      set_err("Compression failed. File closed anyway.",-2);
      return;
    }
    
  }
  
}

int binary_out_file::bool_out(bool dat, std::string name) {
  if (dat) char_out('t');
  else char_out('f');
  return 0;
}

int binary_out_file::char_out(char dat, std::string name) {
  outs.write((char *)&dat,sizeof(char));
  return 0;
}

int binary_out_file::double_out(double dat, std::string name) {
  outs.write((char *)&dat,sizeof(double));
  return 0;
}

int binary_out_file::float_out(float dat, std::string name) {
  outs.write((char *)&dat,sizeof(float));
  return 0;
}

int binary_out_file::int_out(int dat, std::string name) {
  outs.write((char *)&dat,sizeof(int));
  return 0;
}

int binary_out_file::long_out(unsigned long int dat, std::string name) {
  outs.write((char *)&dat,sizeof(long));
  return 0;
}

int binary_out_file::word_out(string dat, std::string name) {
  const char *cstr=dat.c_str();
  int len=dat.length();
  outs.write((char *)&len,sizeof(int));
  outs.write(cstr,len);
  return 0;
}

int binary_out_file::string_out(string dat, std::string name) {
  const char *cstr=dat.c_str();
  int len=dat.length();
  outs.write((char *)&len,sizeof(int));
  outs.write(cstr,len);
  return 0;
}

int binary_out_file::end_object() {
  return 0;
}

int binary_out_file::start_object(std::string type, std::string name) {
  word_out(type);
  if (name.length()>0) {
    word_out(name);
  }
  return 0;
}

int binary_out_file::init_file() {
  char ch=((char)1);
  outs.write(&ch,1);
  return 0;
}

int binary_out_file::end_line() {
  return 0;
}

// ------------------------------------------------------------
// binary_in_file functions
// ------------------------------------------------------------

binary_in_file::binary_in_file(string file_name) {
  ins.open(file_name.c_str(),ios::binary);
}

binary_in_file::~binary_in_file() {
  ins.close();
}

int binary_in_file::bool_in(bool &dat, std::string name) {
  char *c=new char;
  ins.read(c,sizeof(char));
  if (!ins) return 1;
  if (*c=='t') dat=true;
  else dat=false;
  delete c;
  return 0;
}

int binary_in_file::char_in(char &dat, std::string name) {
  char *c=new char;
  ins.read(c,sizeof(char));
  if (!ins) return 1;
  dat=*c;
  delete c;
  return 0;
}

int binary_in_file::double_in(double &dat, std::string name) {
  char *c=new char[sizeof(double)];
  double *d;
  ins.read(c,sizeof(double));
  if (!ins) return 1;
  d=(double *)c;
  dat=*d;
  delete[] c;
  return 0;
}

int binary_in_file::float_in(float &dat, std::string name) {
  char *c=new char[sizeof(float)];
  float *d;
  ins.read(c,sizeof(float));
  if (!ins) return 1;
  d=(float *)c;
  dat=*d;
  delete[] c;
  return 0;
}

int binary_in_file::int_in(int &dat, std::string name) {
  char *c=new char[sizeof(int)];
  int *i;
  ins.read(c,sizeof(int));
  if (!ins) return 1;
  i=(int *)c;
  dat=*i;
  delete[] c;
  return 0;
}

int binary_in_file::long_in(unsigned long int &dat, std::string name) {
  char *c=new char[sizeof(long)];
  long *i;
  ins.read(c,sizeof(long));
  if (!ins) return 1;
  i=(long *)c;
  dat=*i;
  delete[] c;
  return 0;
}

int binary_in_file::word_in(string &dat, std::string name) {
  char *c=new char[sizeof(int)], *c2;
  int *i;
  ins.read(c,sizeof(int));
  if (!ins) return 1;
  i=(int *)c;
  c2=new char[*i+1];
  ins.read(c2,(*i)*sizeof(char));
  if (!ins) {
    cout << "Failed in binary_in_file::word_in()." << endl;
    return 2;
  }
  c2[*i]='\0';
  dat=c2;
  delete[] c;
  delete[] c2;
  return 0;
}

int binary_in_file::string_in(string &dat, std::string name) {
  char *c=new char[sizeof(int)], *c2;
  int *i;
  ins.read(c,sizeof(int));
  if (!ins) return 1;
  i=(int *)c;
  c2=new char[*i+1];
  ins.read(c2,(*i)*sizeof(char));
  if (!ins) {
    cout << "Failed in binary_in_file::string_in()." << endl;
    return 2;
  }
  c2[*i]='\0';
  dat=c2;
  delete[] c;
  delete[] c2;
  return 0;
}

int binary_in_file::start_object(std::string &type, std::string &name) {
  if (word_in(type)!=0) return 1;
  if (name.length()>0) {
    if (word_in(name)!=0) {
      cout << "Failed in binary_in_file::start_object()." << endl;
      return 2;
    }
  }
  return 0;
}

int binary_in_file::skip_object() {
  return 0;
}

int binary_in_file::init_file() {
  char ch;
  ins.read(&ch,1);
  return 0;
}

