/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_CONSTANTS_H
#define O2SCL_CONSTANTS_H

#include <cmath>

/** 
    \brief GSL constants in CGS units 

    The CGS units are given below each constant
*/
namespace gsl_cgs {
  /// cm
  const double schwarzchild_radius=2.95325008e5;
  /// cm / s
  const double speed_of_light=2.99792458e10;
  /// cm^3 / g s^2
  const double gravitational_constant=6.673e-8;
  /// g cm^2 / s
  const double plancks_constant_h=6.62606876e-27;
  /// g cm^2 / s
  const double plancks_constant_hbar=1.05457159642e-27;
  /// cm
  const double astronomical_unit=1.49597870691e13;
  /// cm
  const double light_year=9.46053620707e17;
  /// cm
  const double parsec=3.08567758135e18;
  /// cm / s^2
  const double grav_accel=9.80665e2;
  /// g cm^2 / s^2
  const double electron_volt=1.602176462e-12;
  /// g
  const double mass_electron=9.10938188e-28;
  /// g
  const double mass_muon=1.88353109e-25;
  /// g
  const double mass_proton=1.67262158e-24;
  /// g
  const double mass_neutron=1.67492716e-24;
  /// g cm^2 / s^2
  const double rydberg=2.17987190389e-11;
  /// g cm^2 / K s^2
  const double boltzmann=1.3806503e-16;
  /// A cm^2
  const double bohr_magneton=9.27400899e-20;
  /// A cm^2
  const double nuclear_magneton=5.05078317e-23;
  /// A cm^2
  const double electron_magnetic_moment=9.28476362e-20;
  /// A cm^2
  const double proton_magnetic_moment=1.410606633e-22;
  /// g cm^2 / K mol s^2
  const double molar_gas=8.314472e7;
  /// cm^3 / mol
  const double standard_gas_volume=2.2710981e4;
  /// s
  const double minute=6e1;
  /// s
  const double hour=3.6e3;
  /// s
  const double day=8.64e4;
  /// s
  const double week=6.048e5;
  /// cm
  const double inch=2.54e0;
  /// cm
  const double foot=3.048e1;
  /// cm
  const double yard=9.144e1;
  /// cm
  const double mile=1.609344e5;
  /// cm
  const double nautical_mile=1.852e5;
  /// cm
  const double fathom=1.8288e2;
  /// cm
  const double mil=2.54e-3;
  /// cm
  const double point=3.52777777778e-2;
  /// cm
  const double texpoint=3.51459803515e-2;
  /// cm
  const double micron=1e-4;
  /// cm
  const double angstrom=1e-8;
  /// cm^2
  const double hectare=1e8;
  /// cm^2
  const double acre=4.04685642241e7;
  /// cm^2
  const double barn=1e-24;
  /// cm^3
  const double liter=1e3;
  /// cm^3
  const double us_gallon=3.78541178402e3;
  /// cm^3
  const double quart=9.46352946004e2;
  /// cm^3
  const double pint=4.73176473002e2;
  /// cm^3
  const double cup=2.36588236501e2;
  /// cm^3
  const double fluid_ounce=2.95735295626e1;
  /// cm^3
  const double tablespoon=1.47867647813e1;
  /// cm^3
  const double teaspoon=4.92892159375e0;
  /// cm^3
  const double canadian_gallon=4.54609e3;
  /// cm^3
  const double uk_gallon=4.546092e3;
  /// cm / s
  const double miles_per_hour=4.4704e1;
  /// cm / s
  const double kilometers_per_hour=2.77777777778e1;
  /// cm / s
  const double knot=5.14444444444e1;
  /// g
  const double pound_mass=4.5359237e2;
  /// g
  const double ounce_mass=2.8349523125e1;
  /// g
  const double ton=9.0718474e5;
  /// g
  const double metric_ton=1e6;
  /// g
  const double uk_ton=1.0160469088e6;
  /// g
  const double troy_ounce=3.1103475e1;
  /// g
  const double carat=2e-1;
  /// g
  const double unified_atomic_mass=1.66053873e-24;
  /// cm g / s^2
  const double gram_force=9.80665e2;
  /// cm g / s^2
  const double pound_force=4.44822161526e5;
  /// cm g / s^2
  const double kilopound_force=4.44822161526e8;
  /// cm g / s^2
  const double poundal=1.38255e4;
  /// g cm^2 / s^2
  const double calorie=4.1868e7;
  /// g cm^2 / s^2
  const double btu=1.05505585262e10;
  /// g cm^2 / s^2
  const double therm=1.05506e15;
  /// g cm^2 / s^3
  const double horsepower=7.457e9;
  /// g / cm s^2
  const double bar=1e6;
  /// g / cm s^2
  const double std_atmosphere=1.01325e6;
  /// g / cm s^2
  const double torr=1.33322368421e3;
  /// g / cm s^2
  const double meter_of_mercury=1.33322368421e6;
  /// g / cm s^2
  const double inch_of_mercury=3.38638815789e4;
  /// g / cm s^2
  const double inch_of_water=2.490889e3;
  /// g / cm s^2
  const double psi=6.89475729317e4;
  /// g / cm s
  const double poise=1e0;
  /// cm^2 / s
  const double stokes=1e0;
  /// A s / mol
  const double faraday=9.6485341472e4;
  /// A s
  const double electron_charge=1.602176462e-19;
  /// g / A s^2
  const double gauss=1e-1;
  /// cd / cm^2
  const double stilb=1e0;
  /// cd sr
  const double lumen=1e0;
  /// cd sr / cm^2
  const double lux=1e-4;
  /// cd sr / cm^2
  const double phot=1e0;
  /// cd sr / cm^2
  const double footcandle=1.076e-3;
  /// cd sr / cm^2
  const double lambert=1e0;
  /// cd sr / cm^2
  const double footlambert=1.07639104e-3;
  /// 1 / s
  const double curie=3.7e10;
  /// A s / g
  const double roentgen=2.58e-7;
  /// cm^2 / s^2
  const double rad=1e2;
  /// g
  const double solar_mass=1.98892e33;
  /// cm
  const double bohr_radius=5.291772083e-9;
  /// cm g / s^2
  const double newton=1e5;
  /// cm g / s^2
  const double dyne=1e0;
  /// g cm^2 / s^2
  const double joule=1e7;
  /// g cm^2 / s^2
  const double erg=1e0;
  /// g / K^4 s^3
  const double stefan_boltzmann_constant=5.67039934436e-5;
  /// cm^2
  const double thomson_cross_section=6.65245853542e-25;
}

/** 
    \brief GSL constants in CGSM units

    The CGSM units are given below each constant
*/
namespace gsl_cgsm {
  /// cm
  const double schwarzchild_radius=2.95325008e5;
  /// cm / s
  const double speed_of_light=2.99792458e10;
  /// cm^3 / g s^2
  const double gravitational_constant=6.673e-8;
  /// g cm^2 / s
  const double plancks_constant_h=6.62606876e-27;
  /// g cm^2 / s
  const double plancks_constant_hbar=1.05457159642e-27;
  /// cm
  const double astronomical_unit=1.49597870691e13;
  /// cm
  const double light_year=9.46053620707e17;
  /// cm
  const double parsec=3.08567758135e18;
  /// cm / s^2
  const double grav_accel=9.80665e2;
  /// g cm^2 / s^2
  const double electron_volt=1.602176462e-12;
  /// g
  const double mass_electron=9.10938188e-28;
  /// g
  const double mass_muon=1.88353109e-25;
  /// g
  const double mass_proton=1.67262158e-24;
  /// g
  const double mass_neutron=1.67492716e-24;
  /// g cm^2 / s^2
  const double rydberg=2.17987190389e-11;
  /// g cm^2 / K s^2
  const double boltzmann=1.3806503e-16;
  /// abamp cm^2
  const double bohr_magneton=9.27400899e-21;
  /// abamp cm^2
  const double nuclear_magneton=5.05078317e-24;
  /// abamp cm^2
  const double electron_magnetic_moment=9.28476362e-21;
  /// abamp cm^2
  const double proton_magnetic_moment=1.410606633e-23;
  /// g cm^2 / K mol s^2
  const double molar_gas=8.314472e7;
  /// cm^3 / mol
  const double standard_gas_volume=2.2710981e4;
  /// s
  const double minute=6e1;
  /// s
  const double hour=3.6e3;
  /// s
  const double day=8.64e4;
  /// s
  const double week=6.048e5;
  /// cm
  const double inch=2.54e0;
  /// cm
  const double foot=3.048e1;
  /// cm
  const double yard=9.144e1;
  /// cm
  const double mile=1.609344e5;
  /// cm
  const double nautical_mile=1.852e5;
  /// cm
  const double fathom=1.8288e2;
  /// cm
  const double mil=2.54e-3;
  /// cm
  const double point=3.52777777778e-2;
  /// cm
  const double texpoint=3.51459803515e-2;
  /// cm
  const double micron=1e-4;
  /// cm
  const double angstrom=1e-8;
  /// cm^2
  const double hectare=1e8;
  /// cm^2
  const double acre=4.04685642241e7;
  /// cm^2
  const double barn=1e-24;
  /// cm^3
  const double liter=1e3;
  /// cm^3
  const double us_gallon=3.78541178402e3;
  /// cm^3
  const double quart=9.46352946004e2;
  /// cm^3
  const double pint=4.73176473002e2;
  /// cm^3
  const double cup=2.36588236501e2;
  /// cm^3
  const double fluid_ounce=2.95735295626e1;
  /// cm^3
  const double tablespoon=1.47867647813e1;
  /// cm^3
  const double teaspoon=4.92892159375e0;
  /// cm^3
  const double canadian_gallon=4.54609e3;
  /// cm^3
  const double uk_gallon=4.546092e3;
  /// cm / s
  const double miles_per_hour=4.4704e1;
  /// cm / s
  const double kilometers_per_hour=2.77777777778e1;
  /// cm / s
  const double knot=5.14444444444e1;
  /// g
  const double pound_mass=4.5359237e2;
  /// g
  const double ounce_mass=2.8349523125e1;
  /// g
  const double ton=9.0718474e5;
  /// g
  const double metric_ton=1e6;
  /// g
  const double uk_ton=1.0160469088e6;
  /// g
  const double troy_ounce=3.1103475e1;
  /// g
  const double carat=2e-1;
  /// g
  const double unified_atomic_mass=1.66053873e-24;
  /// cm g / s^2
  const double gram_force=9.80665e2;
  /// cm g / s^2
  const double pound_force=4.44822161526e5;
  /// cm g / s^2
  const double kilopound_force=4.44822161526e8;
  /// cm g / s^2
  const double poundal=1.38255e4;
  /// g cm^2 / s^2
  const double calorie=4.1868e7;
  /// g cm^2 / s^2
  const double btu=1.05505585262e10;
  /// g cm^2 / s^2
  const double therm=1.05506e15;
  /// g cm^2 / s^3
  const double horsepower=7.457e9;
  /// g / cm s^2
  const double bar=1e6;
  /// g / cm s^2
  const double std_atmosphere=1.01325e6;
  /// g / cm s^2
  const double torr=1.33322368421e3;
  /// g / cm s^2
  const double meter_of_mercury=1.33322368421e6;
  /// g / cm s^2
  const double inch_of_mercury=3.38638815789e4;
  /// g / cm s^2
  const double inch_of_water=2.490889e3;
  /// g / cm s^2
  const double psi=6.89475729317e4;
  /// g / cm s
  const double poise=1e0;
  /// cm^2 / s
  const double stokes=1e0;
  /// abamp s / mol
  const double faraday=9.6485341472e3;
  /// abamp s
  const double electron_charge=1.602176462e-20;
  /// g / abamp s^2
  const double gauss=1e0;
  /// cd / cm^2
  const double stilb=1e0;
  /// cd sr
  const double lumen=1e0;
  /// cd sr / cm^2
  const double lux=1e-4;
  /// cd sr / cm^2
  const double phot=1e0;
  /// cd sr / cm^2
  const double footcandle=1.076e-3;
  /// cd sr / cm^2
  const double lambert=1e0;
  /// cd sr / cm^2
  const double footlambert=1.07639104e-3;
  /// 1 / s
  const double curie=3.7e10;
  /// abamp s / g
  const double roentgen=2.58e-8;
  /// cm^2 / s^2
  const double rad=1e2;
  /// g
  const double solar_mass=1.98892e33;
  /// cm
  const double bohr_radius=5.291772083e-9;
  /// cm g / s^2
  const double newton=1e5;
  /// cm g / s^2
  const double dyne=1e0;
  /// g cm^2 / s^2
  const double joule=1e7;
  /// g cm^2 / s^2
  const double erg=1e0;
  /// g / K^4 s^3
  const double stefan_boltzmann_constant=5.67039934436e-5;
  /// cm^2
  const double thomson_cross_section=6.65245853542e-25;
}

/** 
    \brief GSL constants in MKS units

    The MKS units are given below each constant
*/
namespace gsl_mks {
  /// m
  const double schwarzchild_radius=2.95325008e3;
  /// m / s
  const double speed_of_light=2.99792458e8;
  /// m^3 / kg s^2
  const double gravitational_constant=6.673e-11;
  /// kg m^2 / s
  const double plancks_constant_h=6.62606876e-34;
  /// kg m^2 / s
  const double plancks_constant_hbar=1.05457159642e-34;
  /// m
  const double astronomical_unit=1.49597870691e11;
  /// m
  const double light_year=9.46053620707e15;
  /// m
  const double parsec=3.08567758135e16;
  /// m / s^2
  const double grav_accel=9.80665e0;
  /// kg m^2 / s^2
  const double electron_volt=1.602176462e-19;
  /// kg
  const double mass_electron=9.10938188e-31;
  /// kg
  const double mass_muon=1.88353109e-28;
  /// kg
  const double mass_proton=1.67262158e-27;
  /// kg
  const double mass_neutron=1.67492716e-27;
  /// kg m^2 / s^2
  const double rydberg=2.17987190389e-18;
  /// kg m^2 / K s^2
  const double boltzmann=1.3806503e-23;
  /// A m^2
  const double bohr_magneton=9.27400899e-24;
  /// A m^2
  const double nuclear_magneton=5.05078317e-27;
  /// A m^2
  const double electron_magnetic_moment=9.28476362e-24;
  /// A m^2
  const double proton_magnetic_moment=1.410606633e-26;
  /// kg m^2 / K mol s^2
  const double molar_gas=8.314472e0;
  /// m^3 / mol
  const double standard_gas_volume=2.2710981e-2;
  /// s
  const double minute=6e1;
  /// s
  const double hour=3.6e3;
  /// s
  const double day=8.64e4;
  /// s
  const double week=6.048e5;
  /// m
  const double inch=2.54e-2;
  /// m
  const double foot=3.048e-1;
  /// m
  const double yard=9.144e-1;
  /// m
  const double mile=1.609344e3;
  /// m
  const double nautical_mile=1.852e3;
  /// m
  const double fathom=1.8288e0;
  /// m
  const double mil=2.54e-5;
  /// m
  const double point=3.52777777778e-4;
  /// m
  const double texpoint=3.51459803515e-4;
  /// m
  const double micron=1e-6;
  /// m
  const double angstrom=1e-10;
  /// m^2
  const double hectare=1e4;
  /// m^2
  const double acre=4.04685642241e3;
  /// m^2
  const double barn=1e-28;
  /// m^3
  const double liter=1e-3;
  /// m^3
  const double us_gallon=3.78541178402e-3;
  /// m^3
  const double quart=9.46352946004e-4;
  /// m^3
  const double pint=4.73176473002e-4;
  /// m^3
  const double cup=2.36588236501e-4;
  /// m^3
  const double fluid_ounce=2.95735295626e-5;
  /// m^3
  const double tablespoon=1.47867647813e-5;
  /// m^3
  const double teaspoon=4.92892159375e-6;
  /// m^3
  const double canadian_gallon=4.54609e-3;
  /// m^3
  const double uk_gallon=4.546092e-3;
  /// m / s
  const double miles_per_hour=4.4704e-1;
  /// m / s
  const double kilometers_per_hour=2.77777777778e-1;
  /// m / s
  const double knot=5.14444444444e-1;
  /// kg
  const double pound_mass=4.5359237e-1;
  /// kg
  const double ounce_mass=2.8349523125e-2;
  /// kg
  const double ton=9.0718474e2;
  /// kg
  const double metric_ton=1e3;
  /// kg
  const double uk_ton=1.0160469088e3;
  /// kg
  const double troy_ounce=3.1103475e-2;
  /// kg
  const double carat=2e-4;
  /// kg
  const double unified_atomic_mass=1.66053873e-27;
  /// kg m / s^2
  const double gram_force=9.80665e-3;
  /// kg m / s^2
  const double pound_force=4.44822161526e0;
  /// kg m / s^2
  const double kilopound_force=4.44822161526e3;
  /// kg m / s^2
  const double poundal=1.38255e-1;
  /// kg m^2 / s^2
  const double calorie=4.1868e0;
  /// kg m^2 / s^2
  const double btu=1.05505585262e3;
  /// kg m^2 / s^2
  const double therm=1.05506e8;
  /// kg m^2 / s^3
  const double horsepower=7.457e2;
  /// kg / m s^2
  const double bar=1e5;
  /// kg / m s^2
  const double std_atmosphere=1.01325e5;
  /// kg / m s^2
  const double torr=1.33322368421e2;
  /// kg / m s^2
  const double meter_of_mercury=1.33322368421e5;
  /// kg / m s^2
  const double inch_of_mercury=3.38638815789e3;
  /// kg / m s^2
  const double inch_of_water=2.490889e2;
  /// kg / m s^2
  const double psi=6.89475729317e3;
  /// kg m^-1 s^-1
  const double poise=1e-1;
  /// m^2 / s
  const double stokes=1e-4;
  /// A s / mol
  const double faraday=9.6485341472e4;
  /// A s
  const double electron_charge=1.602176462e-19;
  /// kg / A s^2
  const double gauss=1e-4;
  /// cd / m^2
  const double stilb=1e4;
  /// cd sr
  const double lumen=1e0;
  /// cd sr / m^2
  const double lux=1e0;
  /// cd sr / m^2
  const double phot=1e4;
  /// cd sr / m^2
  const double footcandle=1.076e1;
  /// cd sr / m^2
  const double lambert=1e4;
  /// cd sr / m^2
  const double footlambert=1.07639104e1;
  /// 1 / s
  const double curie=3.7e10;
  /// A s / kg
  const double roentgen=2.58e-4;
  /// m^2 / s^2
  const double rad=1e-2;
  /// kg
  const double solar_mass=1.98892e30;
  /// m
  const double bohr_radius=5.291772083e-11;
  /// kg m / s^2
  const double newton=1e0;
  /// kg m / s^2
  const double dyne=1e-5;
  /// kg m^2 / s^2
  const double joule=1e0;
  /// kg m^2 / s^2
  const double erg=1e-7;
  /// kg / K^4 s^3
  const double stefan_boltzmann_constant=5.67039934436e-8;
  /// m^2
  const double thomson_cross_section=6.65245853542e-29;
  /// A^2 s^4 / kg m^3
  const double vacuum_permittivity=8.854187817e-12;
  /// kg m / A^2 s^2
  const double vacuum_permeability=1.25663706144e-6;
}

/** 
    \brief GSL constants in MKSA units

    The MKSA units are given below each constant
*/
namespace gsl_mksa {
  /// m
  const double schwarzchild_radius=2.95325008e3;
  /// m / s
  const double speed_of_light=2.99792458e8;
  /// m^3 / kg s^2
  const double gravitational_constant=6.673e-11;
  /// kg m^2 / s
  const double plancks_constant_h=6.62606876e-34;
  /// kg m^2 / s
  const double plancks_constant_hbar=1.05457159642e-34;
  /// m
  const double astronomical_unit=1.49597870691e11;
  /// m
  const double light_year=9.46053620707e15;
  /// m
  const double parsec=3.08567758135e16;
  /// m / s^2
  const double grav_accel=9.80665e0;
  /// kg m^2 / s^2
  const double electron_volt=1.602176462e-19;
  /// kg
  const double mass_electron=9.10938188e-31;
  /// kg
  const double mass_muon=1.88353109e-28;
  /// kg
  const double mass_proton=1.67262158e-27;
  /// kg
  const double mass_neutron=1.67492716e-27;
  /// kg m^2 / s^2
  const double rydberg=2.17987190389e-18;
  /// kg m^2 / K s^2
  const double boltzmann=1.3806503e-23;
  /// A m^2
  const double bohr_magneton=9.27400899e-24;
  /// A m^2
  const double nuclear_magneton=5.05078317e-27;
  /// A m^2
  const double electron_magnetic_moment=9.28476362e-24;
  /// A m^2
  const double proton_magnetic_moment=1.410606633e-26;
  /// kg m^2 / K mol s^2
  const double molar_gas=8.314472e0;
  /// m^3 / mol
  const double standard_gas_volume=2.2710981e-2;
  /// s
  const double minute=6e1;
  /// s
  const double hour=3.6e3;
  /// s
  const double day=8.64e4;
  /// s
  const double week=6.048e5;
  /// m
  const double inch=2.54e-2;
  /// m
  const double foot=3.048e-1;
  /// m
  const double yard=9.144e-1;
  /// m
  const double mile=1.609344e3;
  /// m
  const double nautical_mile=1.852e3;
  /// m
  const double fathom=1.8288e0;
  /// m
  const double mil=2.54e-5;
  /// m
  const double point=3.52777777778e-4;
  /// m
  const double texpoint=3.51459803515e-4;
  /// m
  const double micron=1e-6;
  /// m
  const double angstrom=1e-10;
  /// m^2
  const double hectare=1e4;
  /// m^2
  const double acre=4.04685642241e3;
  /// m^2
  const double barn=1e-28;
  /// m^3
  const double liter=1e-3;
  /// m^3
  const double us_gallon=3.78541178402e-3;
  /// m^3
  const double quart=9.46352946004e-4;
  /// m^3
  const double pint=4.73176473002e-4;
  /// m^3
  const double cup=2.36588236501e-4;
  /// m^3
  const double fluid_ounce=2.95735295626e-5;
  /// m^3
  const double tablespoon=1.47867647813e-5;
  /// m^3
  const double teaspoon=4.92892159375e-6;
  /// m^3
  const double canadian_gallon=4.54609e-3;
  /// m^3
  const double uk_gallon=4.546092e-3;
  /// m / s
  const double miles_per_hour=4.4704e-1;
  /// m / s
  const double kilometers_per_hour=2.77777777778e-1;
  /// m / s
  const double knot=5.14444444444e-1;
  /// kg
  const double pound_mass=4.5359237e-1;
  /// kg
  const double ounce_mass=2.8349523125e-2;
  /// kg
  const double ton=9.0718474e2;
  /// kg
  const double metric_ton=1e3;
  /// kg
  const double uk_ton=1.0160469088e3;
  /// kg
  const double troy_ounce=3.1103475e-2;
  /// kg
  const double carat=2e-4;
  /// kg
  const double unified_atomic_mass=1.66053873e-27;
  /// kg m / s^2
  const double gram_force=9.80665e-3;
  /// kg m / s^2
  const double pound_force=4.44822161526e0;
  /// kg m / s^2
  const double kilopound_force=4.44822161526e3;
  /// kg m / s^2
  const double poundal=1.38255e-1;
  /// kg m^2 / s^2
  const double calorie=4.1868e0;
  /// kg m^2 / s^2
  const double btu=1.05505585262e3;
  /// kg m^2 / s^2
  const double therm=1.05506e8;
  /// kg m^2 / s^3
  const double horsepower=7.457e2;
  /// kg / m s^2
  const double bar=1e5;
  /// kg / m s^2
  const double std_atmosphere=1.01325e5;
  /// kg / m s^2
  const double torr=1.33322368421e2;
  /// kg / m s^2
  const double meter_of_mercury=1.33322368421e5;
  /// kg / m s^2
  const double inch_of_mercury=3.38638815789e3;
  /// kg / m s^2
  const double inch_of_water=2.490889e2;
  /// kg / m s^2
  const double psi=6.89475729317e3;
  /// kg m^-1 s^-1
  const double poise=1e-1;
  /// m^2 / s
  const double stokes=1e-4;
  /// A s / mol
  const double faraday=9.6485341472e4;
  /// A s
  const double electron_charge=1.602176462e-19;
  /// kg / A s^2
  const double gauss=1e-4;
  /// cd / m^2
  const double stilb=1e4;
  /// cd sr
  const double lumen=1e0;
  /// cd sr / m^2
  const double lux=1e0;
  /// cd sr / m^2
  const double phot=1e4;
  /// cd sr / m^2
  const double footcandle=1.076e1;
  /// cd sr / m^2
  const double lambert=1e4;
  /// cd sr / m^2
  const double footlambert=1.07639104e1;
  /// 1 / s
  const double curie=3.7e10;
  /// A s / kg
  const double roentgen=2.58e-4;
  /// m^2 / s^2
  const double rad=1e-2;
  /// kg
  const double solar_mass=1.98892e30;
  /// m
  const double bohr_radius=5.291772083e-11;
  /// kg m / s^2
  const double newton=1e0;
  /// kg m / s^2
  const double dyne=1e-5;
  /// kg m^2 / s^2
  const double joule=1e0;
  /// kg m^2 / s^2
  const double erg=1e-7;
  /// kg / K^4 s^3
  const double stefan_boltzmann_constant=5.67039934436e-8;
  /// m^2
  const double thomson_cross_section=6.65245853542e-29;
  /// A^2 s^4 / kg m^3
  const double vacuum_permittivity=8.854187817e-12;
  /// kg m / A^2 s^2
  const double vacuum_permeability=1.25663706144e-6;
}

/** 
    \brief GSL numerical constants

 */
namespace gsl_num {
  const double yotta=1e24;
  const double zetta=1e21;
  const double exa=1e18;
  const double peta=1e15;
  const double tera=1e12;
  const double giga=1e9;
  const double mega=1e6;
  const double kilo=1e3;
  const double milli=1e-3;
  const double micro=1e-6;
  const double nano=1e-9;
  const double pico=1e-12;
  const double femto=1e-15;
  const double atto=1e-18;
  const double zepto=1e-21;
  const double yocto=1e-24;

  /** 
      \brief Fine structure constant (updated from 
      http://physics.nist.gov/cuu/Constants)
  */
  const double fine_structure=7.2973525376e-3;
  /** 
      \brief Avogadro's number (updated from 
      http://physics.nist.gov/cuu/Constants)
  */
  const double avogadro=6.02214179e23;
}

/** 
    
\brief O2scl constants


 */
namespace o2scl_const {
  /// \f$ \pi \f$ 
  const double pi=acos(-1.0);
  /// \f$ \pi^2 \f$ 
  const double pi2=pi*pi;
  /// \f$ \zeta(3/2) \f$
  const double zeta32=2.6123753486854883433;
  /// \f$ \zeta(2) \f$
  const double zeta2=1.6449340668482264365;
  /// \f$ \zeta(5/2) \f$
  const double zeta52=1.3414872572509171798;
  /// \f$ \zeta(3) \f$
  const double zeta3=1.2020569031595942854;
  /// \f$ \zeta(5) \f$
  const double zeta5=1.0369277551433699263;
  /// \f$ \zeta(7) \f$

  const double zeta7=1.0083492773819228268;

  /** \name Particle Physics Booklet 

      (see also D.E. Groom, et. al., Euro. Phys. J. C 15 (2000) 1.)
  */
  //@{
  /// \f$ \sin^2 \theta_W \f$
  const double sin2_theta_weak=0.2224;
  /// 1 MeV in kg
  const double mev_kg=1.782661731e-30; 
  /// 1 eV in \f$ kg \cdot m^2 /s^2 \f$ (Joules)
  const double ev_mks=1.602176462e-19; 
  /// 1 MeV in \f$ g \cdot cm^2 / s^2 \f$ (ergs)
  const double mev_cgs=1.60217733e-6; 
  /// 1 MeV in Kelvin
  const double boltzmann_mev_K=8.617342e-11; 
  //@}

  /** \name From http://physics.nist.gov/cuu/Constants */
  //@{
  /// \f$ \hbar c \f$ in MeV fm
  const double hc_mev_fm=197.3269631; 
  /// Fermi coupling constant (\f$ G_F \f$) in \f$ GeV^{-2} \f$
  const double gfermi_gev=1.16637e-5; 
  /// \f$ \hbar c \f$ in MeV cm
  const double hc_mev_cm=1.973269631e-11; 
  //@}

  /** \name Squared electron charge
   */
  //@{
  /** 
      \brief Electron charge squared in Gaussian units

      In Gaussian Units:
      \f{eqnarray*}
      &\vec{\nabla} \cdot \vec{E} = 4 \pi \rho \, ,
      \quad
      \vec{E}=-\vec{\nabla} \Phi \, ,
      \quad
      \nabla^2 \Phi = - 4 \pi \rho \, ,
      &\\&
      F=\frac{q_1 q_2}{r^2} \, ,
      \quad
      W=\frac{1}{2} \int \rho V d^3 x
      =\frac{1}{8 \pi} \int | \vec{E} |^2 d^3 x \, ,
      \quad 
      \alpha=\frac{e^2}{\hbar c}=\frac{1}{137}&
      \f}
  */
  const double e2_gaussian=o2scl_const::hc_mev_fm*gsl_num::fine_structure;
  /** \brief Electron charge sqaured in 
      Heaviside-Lorentz units where \f$\hbar=c=1\f$.

      In Heaviside-Lorentz units:
      \f{eqnarray*}
      &\vec{\nabla} \cdot \vec{E} = \rho \, ,
      \quad
      \vec{E}=-\vec{\nabla} \Phi \, ,
      \quad
      \nabla^2 \Phi = - \rho \, ,
      &\\&
      F=\frac{q_1 q_2}{4 \pi r^2} \, ,
      \quad
      W=\frac{1}{2} \int \rho V d^3 x
      =\frac{1}{2} \int | \vec{E} |^2 d^3 x \, ,
      \quad
      \alpha=\frac{e^2}{4 \pi}=\frac{1}{137}&
      \f}
  */      
  const double e2_hlorentz=gsl_num::fine_structure*4.0*pi;
  /** 
      \brief Electron charge squared in SI(MKSA) units

      In MKSA units:
      \f{eqnarray*}
      &\vec{\nabla} \cdot \vec{E} = \rho \, ,
      \quad
      \vec{E}=-\vec{\nabla} \Phi \, ,
      \quad
      \nabla^2 \Phi = - \rho \, ,
      &\\&
      F=\frac{1}{4 \pi \varepsilon_0}\frac{q_1 q_2}{r^2} \, ,
      \quad
      W=\frac{1}{2} \int \rho V d^3 x
      =\frac{\varepsilon_0}{2} \int | \vec{E} |^2 d^3 x \, ,
      \quad
      \alpha=\frac{e^2}{4 \pi \varepsilon_0 \hbar c}=\frac{1}{137}&
      \f}

      Note the conversion formulas
      \f[
      q_HL=\sqrt{4 \pi} q_G = \frac{1}{\sqrt{\varepsilon_0}} q_{SI}
      \f]
      as mentioned in pg. 13 of D. Griffiths Intro to Elem. Particles.
  */      
  const double e2_mksa=gsl_mksa::electron_charge;
  //@}

}

/** 
    \brief Constants in units of fm
    
    In nuclear physics is frequently convenient to work in units of
    \f$\mathrm{fm}\f$ with \f$\hbar=c=k_B=1\f$. Several useful
    constants are given here.

    For example, \ref mev gives 1 \f$\mathrm{MeV}\f$ in units of
    \f$\mathrm{fm}^{-1}\f$ (the solution to the equation \f$ 1
    \mathrm{MeV} = x~\mathrm{fm}^{-1}\f$). If you have a number in
    \f$\mathrm{MeV}\f$, you can multiply by \ref mev to get a number
    in units of \f$\mathrm{fm}^{-1}\f$. Alternatively, \ref mev is a
    number with units \f$\mathrm{MeV}^{-1}\cdot\mathrm{fm}^{-1}\f$.
    These can be combined, so that \ref erg divided by \ref sec is
    1 \f$\mathrm{erg/sec}\f$ in units of \f$\mathrm{fm}^{-2}\f$.
    
*/
namespace o2scl_fm {
  /** \name Masses from Particle Physics Booklet 

  (see also D.E. Groom, et. al., Euro. Phys. J. C 15 (2000) 1.)
  */
  //@{
  /// Electron mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_electron=0.510998902/o2scl_const::hc_mev_fm; 
  /// Muon mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_muon=105.658357/o2scl_const::hc_mev_fm;
  /// Atomic mass unit in \f$ \mathrm{fm}^{-1} \f$
  const double mass_amu=931.494013/o2scl_const::hc_mev_fm; 
  /// Neutron mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_neutron=939.565/o2scl_const::hc_mev_fm;
  /// Proton mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_proton=938.272/o2scl_const::hc_mev_fm; 
  /// \f$\Lambda\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_lambda=1115.683/o2scl_const::hc_mev_fm;
  /// \f$\Sigma^{-}\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_sigmam=1197.45/o2scl_const::hc_mev_fm;
  /// \f$\Sigma^{0}\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_sigma=1192.642/o2scl_const::hc_mev_fm;
  /// \f$\Sigma^{+}\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_sigmap=1189.37/o2scl_const::hc_mev_fm;
  /// \f$\Xi^{-}\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_cascadem=1321.3/o2scl_const::hc_mev_fm;
  /// \f$\Xi^{0}\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_cascade=1314.8/o2scl_const::hc_mev_fm;
  /// \f$\omega\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_omega=782.57/o2scl_const::hc_mev_fm;
  /// \f$\rho\f$ mass in \f$ \mathrm{fm}^{-1} \f$
  const double mass_rho=769.3/o2scl_const::hc_mev_fm;
  //@}

  /** \name www.nist.gov
   */
  //@{
  /** 
      \brief Alpha particle mass in \f$ \mathrm{fm}^{-1} \f$

      This does not include the mass of the additional two electrons
      which are present in a helium atom.
  */
  const double mass_alpha=3727.37905/o2scl_const::hc_mev_fm;
  //@}
  
  /// 1 \f$\mathrm{MeV}\f$ in \f$\mathrm{fm}^{-1}\f$
  const double mev=1.0/o2scl_const::hc_mev_fm;

  /// 1 \f$\mathrm{kg}\f$ in \f$\mathrm{fm}^{-1}\f$
  const double kg=mev/1.782661731e-30;
  
  /// 1 \f$M_{\odot}/km^{3}\f$ in \f$\mathrm{fm}^{-4}\f$
  const double msun_per_km3=gsl_mks::solar_mass/1.0e54*kg; 

  /// 1 \f$\mathrm{Kelvin}\f$ in \f$\mathrm{fm}^{-1}\f$
  const double Kelvin=8.617342e-11*mev;

  /// 1 \f$\mathrm{Joule}\f$ in \f$\mathrm{fm}^{-1}\f$
  const double joule=kg/gsl_mks::speed_of_light/gsl_mks::speed_of_light;
  
  /// 1 \f$\mathrm{erg}\f$ in \f$\mathrm{fm}^{-1}\f$
  const double erg=kg/1.0e3/gsl_cgs::speed_of_light/gsl_cgs::speed_of_light;
  
  /// 1 \f$\mathrm{second}\f$ in \f$\mathrm{fm}\f$
  const double sec=gsl_mks::speed_of_light*1.0e15;

}


#endif
