/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/err_hnd.h>
#include <o2scl/string_conv.h>

using namespace std;
using namespace o2scl;

err_class *err_class::ptr=0;
err_class *o2scl::err_hnd;
err_class o2scl::def_err_hnd;

err_class::err_class() {
  ptr=this;
  err_hnd=this;
  gsl_set_error_handler(err_hnd->gsl_hnd);

  a_errno=0;
  a_line=0;
  a_file=0;
  a_reason[0]='\0';

  mode=0;
  array_abort=true;

  fname_size=35;
}

void err_class::reset() {

  a_errno=0;
  a_line=0;
  a_file=0;
  a_reason[0]='\0';

  return;
}

void err_class::gsl_hnd(const char *reason, const char *file, int line, 
			int lerrno) {
  ptr->a_errno=lerrno;
  ptr->a_file=(char *)file;
  ptr->a_line=line;
  strncpy(ptr->a_reason,reason,200);
  
  if (ptr->mode>=1) {
    cout << ptr->get_str() << endl;
    if (ptr->mode==2) exit(lerrno);
  }
  return;
}

void err_class::set(const char *reason, const char *file, 
		    int line, int lerrno) {
  a_errno=lerrno;
  a_file=(char *)file;
  a_line=line;
  strncpy(a_reason,reason,200);

#ifdef O2SCL_ARRAY_ABORT
  if (array_abort && lerrno==gsl_index) {
    cout << "O2scl: Fatal array error." << endl;
    cout << get_str() << endl;
    exit(lerrno);
  }
#endif
  
  if (mode>=1) {
    cout << get_str() << endl;
    if (ptr->mode==2) exit(lerrno);
  }
  return;
}

void err_class::add(const char *reason, const char *file, 
		    int line, int lerrno) {
  if (a_errno==0) {
    a_errno=lerrno;
    a_file=(char *)file;
    a_line=line;
    strncpy(a_reason,reason,200);
  } else {
    a_errno=lerrno;
    a_file=(char *)file;
    a_line=line;

    // Copy old a_reason to tmp
    char tmp[rsize];
    strcpy(tmp,a_reason);

    // Fill temp with reason plus old reason from tmp
    std::string temp=reason;
    temp+="\n ";
    temp+=tmp;

    // Copy temp to a_reason
    size_t i;
    for(i=0;i<temp.size() && i<rsize-1;i++) {
      a_reason[i]=temp[i];
    }

    // Add string termination character
    a_reason[i]='\0';
  }

#ifdef O2SCL_ARRAY_ABORT
  if (array_abort && lerrno==gsl_index) {
    cout << "O2scl: Fatal array error." << endl;
    cout << get_str() << endl;
    exit(lerrno);
  }
#endif

  if (mode==2) {
    cout << get_str() << endl;
    exit(lerrno);
  }
  return;
}

void err_class::get(const char *&reason, const char *&file, 
		    int &line, int &lerrno) {

  lerrno=a_errno;
  file=a_file;
  reason=a_reason;
  line=a_line;

  return;
}

int err_class::get_errno() {
  return a_errno;
}

int err_class::get_line() {
  return a_line;
}

const char *err_class::get_reason() {
  return a_reason;
}

const char *err_class::get_file() {
  return a_file;
}

const char *err_class::get_str() {
  if (a_errno==0) {
    sprintf(fullstr,"No error occured.");
  } else {
    std::string temp="Error number ";
    temp+=itos(a_errno);
    temp+=" in file ";

    // Print out only the last part of the filename to keep it on 
    // only 1 line
    std::string ftemp=a_file;
    if (ftemp.length()>fname_size) {
      ftemp=ftemp.substr(ftemp.length()-fname_size,ftemp.length());
      ftemp=((std::string)"...")+ftemp;
    }
    temp+=ftemp;

    temp+=" at line ";
    temp+=itos(a_line);
    temp+=".\n ";
    temp+=a_reason;
    size_t i;
    // Use fsize-1 here because we want to save
    // space for the string termination character
    for(i=0;i<temp.size() && i<fsize-1;i++) {
      fullstr[i]=temp[i];
    }
    fullstr[i]='\0';
  }
  return fullstr;
}

