/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_FILE_FORMAT_H
#define O2SCL_FILE_FORMAT_H

#include <iostream>
#include <fstream>
#include <sstream>

#include <o2scl/lib_settings.h>

#ifndef DOXYGENP
namespace o2scl {
#endif
  
  /** 
      \brief Make a temporary file
  */
  int o2scl_make_tfile(std::string &fname, int st=0);

  /** 
      \brief Class for output file formats [abstract base]
  */
  class out_file_format {
  public:

    virtual ~out_file_format() {}

    /** \brief Output a bool variable
     */
    virtual int bool_out(bool dat, std::string name="")=0;

    /** \brief Output a char variable
     */
    virtual int char_out(char dat, std::string name="")=0;

    /** \brief Output a float variable
     */
    virtual int float_out(float dat, std::string name="")=0;

    /** \brief Output a double variable
     */
    virtual int double_out(double dat, std::string name="")=0;

    /** \brief Output an int variable
     */
    virtual int int_out(int dat, std::string name="")=0;

    /** \brief Output an long variable
     */
    virtual int long_out(unsigned long int dat, std::string name="")=0;

    /** \brief Output a string
     */
    virtual int string_out(std::string dat, std::string name="")=0;

    /** \brief Output a word
     */
    virtual int word_out(std::string dat, std::string name="")=0;

    /** \brief Start object output
     */
    virtual int start_object(std::string type, std::string name="")=0;

    /** \brief End object output
     */
    virtual int end_object()=0;

    /** \brief End a line of output
     */
    virtual int end_line()=0;

    /** \brief Output initialization */
    virtual int init_file()=0;
    
    /// Finish the file
    virtual int clean_up()=0;
    
  };
  
  /** 
      \brief Class for input file formats [abstract base]
   */
  class in_file_format {
  public:

    virtual ~in_file_format() {}

    /** \brief Input a bool variable
     */
    virtual int bool_in(bool &dat, std::string name="")=0;

    /** \brief Input a char variable
     */
    virtual int char_in(char &dat, std::string name="")=0;

    /** \brief Input a double variable
     */
    virtual int double_in(double &dat, std::string name="")=0;

    /** \brief Input a float variable
     */
    virtual int float_in(float &dat, std::string name="")=0;

    /** \brief Input an int variable
     */
    virtual int int_in(int &dat, std::string name="")=0;

    /** \brief Input an long variable
     */
    virtual int long_in(unsigned long int &dat, std::string name="")=0;

    /** \brief Input a string variable
     */
    virtual int string_in(std::string &dat, std::string name="")=0;

    /** \brief Input a word variable
     */
    virtual int word_in(std::string &dat, std::string name="")=0;
    
    /// Start object input
    virtual int start_object(std::string &type, std::string &name)=0;
    
    /** \brief Skip the present object for the next call to read_type().
     */
    virtual int skip_object()=0;
    
    /// End object input
    virtual int end_object()=0;

    /** \brief Read initialization */
    virtual int init_file()=0;

    /// Finish file input
    virtual int clean_up()=0;
  };

}

#endif
