/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/bps_eos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;

#include <o2scl/test_mgr.h>
#include <o2scl/table.h>
#include <o2scl/eos_ioc.h>

const double xmev_kg=1.782661731e-30;
const double mev_per_fm3_msun_km3=xmev_kg/gsl_mks::solar_mass*1.0e54;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  thermo th;
  bps_eos be;
  int Z, A;
  double conv=mev_per_fm3_msun_km3;
  double cf=(gsl_cgs::mass_neutron+gsl_cgs::mass_proton)/2.0*1.0e39;
  double barn, ed_bps, pr_bps, pr;

  int sz,sz2;
  table *bps;
  string fname=lib_settings.get_data_dir()+"bps.eos", tname;
  
  /// Instantiate I/O objects
  bps_eos_io_type bps_io;
  hadronic_eos_io_type hadronic_eos_io;
  eos_io_type eos_io;
  table_io_type table_io;

  text_in_file *tif=new text_in_file(fname);
  collection co;
  void *vp;
  co.in_one(tif,"table",tname,vp,sz,sz2);
  bps=(table *)vp;
  delete tif;
  
  // Cubic spline interpolation doesn't do very well here, so we use
  // linear interpolation
  linear_interp<ovector_view> i1;
  linear_interp<ovector_const_subvector> i2;
  bps->set_interp(i1,i2);

  for(pr=8.0e-10/conv/hc_mev_fm;pr>=1.0e-16/conv/hc_mev_fm;pr/=2.0) {
    th.pr=pr;
    be.calc_pressure(th,barn,Z,A);
    // Output energy and pressure in MeV/fm^3
    cout << th.ed*hc_mev_fm << " " << th.pr*hc_mev_fm
	 << " " << barn*cf << " " << A << " " << Z << " ";
    
    // Convert from g/cm^3 and dyn/cm^2 to MeV/fm^3
    ed_bps=bps->interp("nb",barn*1.0e39,"rho")/
      mev_cgs*pow(gsl_cgs::speed_of_light,2.0)/1.0e39;
    pr_bps=bps->interp("nb",barn*1.0e39,"P")/
      mev_cgs/1.0e39;

    cout << (ed_bps-th.ed*hc_mev_fm)/ed_bps << " " 
	 << (pr_bps-th.pr*hc_mev_fm)/pr_bps << endl;

    t.test_rel((ed_bps-th.ed*hc_mev_fm)/ed_bps,0.0,2.0e-3,"Energy");
    t.test_rel((pr_bps-th.pr*hc_mev_fm)/pr_bps,0.0,1.5e-1,"Pressure");
  }
  cout << endl;

  for(barn=7.0e11/cf;barn>=7.0e6/cf;barn/=2.0) {
    be.calc_density(barn,th,Z,A);
    
    ed_bps=bps->interp("nb",barn*1.0e39,"rho")/
      mev_cgs*pow(gsl_cgs::speed_of_light,2.0)/1.0e39;
    pr_bps=bps->interp("nb",barn*1.0e39,"P")/
      mev_cgs/1.0e39;
    
    cout << th.ed*hc_mev_fm*conv << " " << th.pr*hc_mev_fm*conv 
	 << " " << barn*cf << " " << A << " " << Z << " ";
    cout << (ed_bps-th.ed*hc_mev_fm)/ed_bps << " " 
	 << (pr_bps-th.pr*hc_mev_fm)/pr_bps << endl;
    
    t.test_rel((ed_bps-th.ed*hc_mev_fm)/ed_bps,0.0,2.0e-3,"Energy");
    t.test_rel((pr_bps-th.pr*hc_mev_fm)/pr_bps,0.0,1.5e-1,"Pressure");
  }
  cout << endl;

  mnmsk_mass mm;
  be.set_mass_formula(mm);
  for(barn=7.0e11/cf;barn>=7.0e6/cf;barn/=1.2) {
    be.calc_density(barn,th,Z,A);
    
    ed_bps=bps->interp("nb",barn*1.0e39,"rho")/
      mev_cgs*pow(gsl_cgs::speed_of_light,2.0)/1.0e39;
    pr_bps=bps->interp("nb",barn*1.0e39,"P")/
      mev_cgs/1.0e39;
    
    cout << th.ed*hc_mev_fm*conv << " " << th.pr*hc_mev_fm*conv 
	 << " " << barn*cf << " " << A << " " << Z << " ";
    cout << (ed_bps-th.ed*hc_mev_fm)/ed_bps << " " 
	 << (pr_bps-th.pr*hc_mev_fm)/pr_bps << endl;
  }
  
  for(int tA=120;tA>=50;tA-=10) {
    be.calc_density_fixedA(4.0e11/cf,th,Z,tA);
    cout << tA << " " << Z << endl;
  }
  for(int tA=49;tA>=40;tA--) {
    be.calc_density_fixedA(4.0e11/cf,th,Z,tA);
    cout << tA << " " << Z << endl;
  }
  cout << mm.binding_energy(36,82)/118.0
       << " " << mm.binding_energy(12,28)/40.0 << endl;

  t.report();
  return 0;
}

