#include <o2scl/cfl6_eos_old.h>
#include <o2scl/bag_eos.h>
#include <o2scl/eff_quark.h>
#include <o2scl/gsl_mroot_hybrids.h>
#include <o2scl/gsl_inte_qng.h>
#include <o2scl/gsl_mroot_hybrids.h>
#include <o2scl/cern_mroot.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  system("date");

  cout.setf(ios::scientific);

  cfl6_eos_old cfl;

  // Use a faster integration routine
  gsl_inte_qng<void *,funct<void *> > inew;
  cfl.set_inte(inew);

  //  cern_mroot<void *,mm_funct<void *> > cm;
  //  cfl.set_solver(cm);

  quark u(cfl.up_default_mass,6.0);
  quark d(cfl.down_default_mass,6.0);
  quark s(cfl.strange_default_mass,6.0);

  // Test new version of calc_temp_p().
  cfl.zerot=true;
  cfl.fixed_mass=false;
  cfl.color_neut=true;
  cfl.gap_limit=-0.01;
  cfl.set_parameters();
  cout << cfl.L << " " << cfl.G << " " << cfl.K << " "
       << cfl.GD << " " << cfl.KD << endl;
  u.del=0.5;
  d.del=0.5;
  s.del=0.5;
  u.qq=-0.05;
  d.qq=-0.05;
  s.qq=-0.5;

  thermo thx;

  double muq=2.7;
  
  u.mu=muq;
  d.mu=muq;
  s.mu=muq;
  int ret2=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
  cout << (u.n+d.n+s.n)/3.0 << " " << ret2 << endl;
  cout << u.qq << " " << d.qq << " " << s.qq << endl;
  cout << u.del << " " << d.del << " " << s.del << endl;
  cout << endl;

  t.test_gen(ret2==0,"err val");
  t.test_rel(u.qq,-0.03406,2.0e-2,"qqu");
  t.test_rel(d.qq,-0.03406,2.0e-2,"qqd");
  t.test_rel(s.qq,-0.31856,2.0e-2,"qqs");
  t.test_rel(u.del,0.38061,2.0e-2,"delu");
  t.test_rel(d.del,0.38061,2.0e-2,"deld");
  t.test_rel(s.del,0.38061,2.0e-2,"dels");
  
  if (false) {
    cfl.KD=cfl.K/2.0;
    ret2=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
    cout << (u.n+d.n+s.n)/3.0 << " " << ret2 << endl;
    cout << u.qq << " " << d.qq << " " << s.qq << endl;
    cout << u.del << " " << d.del << " " << s.del << endl;
    cout << endl;
    
    cfl.KD=-cfl.K/2.0;
    ret2=cfl.calc_temp_p(u,d,s,0.0/hc_mev_fm,thx);
    cout << (u.n+d.n+s.n)/3.0 << " " << ret2 << endl;
    cout << u.qq << " " << d.qq << " " << s.qq << endl;
    cout << u.del << " " << d.del << " " << s.del << endl;
    cout << endl;
  }

  system("date");

  t.report();

  return 0;
}
