#include <o2scl/cfl_njl_eos_old.h>
#include <o2scl/cfl_njl_eos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  // Inits
  
  double ss1,ss2,ss3,gap1,gap2,gap3,n3,n8;
  double ss12,ss22,ss32,gap12,gap22,gap32,n32,n82;
  cfl_njl_eos_old cfl;
  cfl_njl_eos cfl2;
  thermo th, th2;
  quark u(cfl.up_default_mass,6.0);
  quark d(cfl.down_default_mass,6.0);
  quark s(cfl.strange_default_mass,6.0);
  quark u2(cfl.up_default_mass,6.0);
  quark d2(cfl.down_default_mass,6.0);
  quark s2(cfl.strange_default_mass,6.0);
  cfl.set_quarks(u,d,s);
  cfl.set_thermo(th);
  cfl2.set_quarks(u2,d2,s2);
  cfl2.set_thermo(th2);

  // Run test functions
  
  cfl2.test_normal_eigenvalues(t);
  cfl2.test_gapped_eigenvalues(t);

  u2.mu=2.0;
  d2.mu=2.1;
  s2.mu=2.2;
  u2.ms=0.5;
  d2.ms=0.6;
  s2.ms=0.7;
  u2.del=1.0;
  d2.del=1.1;
  s2.del=1.2;
  cfl2.test_derivatives(1.5,0.0,0.0,t);

  cfl2.test_integration(t);

  // Compare new and old

  cfl.set_parameters();
  cfl2.set_parameters();
  t.test_rel(cfl.B0,cfl2.B0,1.0e-6,"B0");
  
  u.mu=2.5;
  d.mu=2.6;
  s.mu=2.7;
  u2.mu=2.5;
  d2.mu=2.6;
  s2.mu=2.7;
  u.qq=-1.0;
  d.qq=-1.1;
  s.qq=-1.2;
  u2.qq=-1.0;
  d2.qq=-1.1;
  s2.qq=-1.2;
  u.del=0.5;
  d.del=0.6;
  s.del=0.7;
  u2.del=0.5;
  d2.del=0.6;
  s2.del=0.7;

  cfl.calc_eq_temp_p(u,d,s,ss1,ss2,ss3,gap1,gap2,gap3,
		     0.0,0.0,n3,n8,th,4.0/hc_mev_fm);
  cfl2.calc_eq_temp_p(u2,d2,s2,ss12,ss22,ss32,gap12,gap22,gap32,
		      0.0,0.0,n32,n82,th2,4.0/hc_mev_fm);
  
  t.test_rel(ss1,ss12,1.0e-6,"ss1");
  t.test_rel(ss2,ss22,1.0e-6,"ss2");
  t.test_rel(ss3,ss32,1.0e-6,"ss3");
  t.test_rel(gap1,gap12,1.0e-6,"gap1");
  t.test_rel(gap2,gap22,1.0e-6,"gap2");
  t.test_rel(gap3,gap32,1.0e-6,"gap3");
  t.test_rel(u.n,u2.n,1.0e-6,"nu");
  t.test_rel(d.n,d2.n,1.0e-6,"nd");
  t.test_rel(s.n,s2.n,1.0e-6,"ns");
  t.test_rel(u.mu,u2.mu,1.0e-6,"muu");
  t.test_rel(d.mu,d2.mu,1.0e-6,"mud");
  t.test_rel(s.mu,s2.mu,1.0e-6,"mus");
  t.test_rel(u.ed,u2.ed,1.0e-6,"edu");
  t.test_rel(d.ed,d2.ed,1.0e-6,"edd");
  t.test_rel(s.ed,s2.ed,1.0e-6,"eds");
  t.test_rel(u.pr,u2.pr,1.0e-6,"pru");
  t.test_rel(d.pr,d2.pr,1.0e-6,"prd");
  t.test_rel(s.pr,s2.pr,1.0e-6,"prs");
  t.test_rel(th.ed,th2.ed,1.0e-6,"thed");
  t.test_rel(th.pr,th2.pr,1.0e-6,"thpr");
  t.test_rel(th.en,th2.en,1.0e-2,"then");

  /// Perform a test of the equal mass case

  cfl2.G=0.2166586;
  cfl2.GD=0.1575699;
  cfl2.K=0.0;
  cfl2.B0=20.07754;
  u2.m=0.0;
  d2.m=0.0;
  s2.m=0.0;
  u.mu=1.32;
  u.del=0.06;
  u.qq=-1.118;
  d.mu=1.32;
  d.del=0.06;
  d.qq=-1.118;
  s.mu=1.32;
  s.del=0.06;
  s.qq=-1.118;

  cfl2.calc_eq_temp_p(u2,d2,s2,ss12,ss22,ss32,gap12,gap22,gap32,
		      0.0,0.0,n32,n82,th2,2.0/hc_mev_fm);
  double dpdqq=ss12;
  double pr1=th2.pr;
  u2.qq+=1.0e-3;
  cfl2.calc_eq_temp_p(u2,d2,s2,ss12,ss22,ss32,gap12,gap22,gap32,
		      0.0,0.0,n32,n82,th2,2.0/hc_mev_fm);
  u2.qq-=1.0e-3;
  double pr2=th2.pr;
  t.test_rel(-dpdqq,(pr2-pr1)/1.0e-3,1.0e-3,"dpdqq");

  cfl2.test_derivatives(1.6,0.0,0.0,t);

  t.report();

  return 0;
}
  

