/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_QUARK_EOS_H
#define O2SCL_QUARK_EOS_H

#include <iostream>
#include <o2scl/eos.h>
#include <o2scl/quark.h>
#include <o2scl/deriv.h>
#include <o2scl/mroot.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Quark matter equation of state base
  */
  class quark_eos : public eos {
  public:
    quark_eos();

    virtual ~quark_eos() {};

    /// Calculate equation of state as a function of chemical potentials
    virtual int calc_p(quark &u, quark &d, quark &s, thermo &th);

    /// Calculate equation of state as a function of density
    virtual int calc_e(quark &u, quark &d, quark &s, thermo &th);

    /// Calculate equation of state as a function of chemical potentials
    virtual int calc_temp_p(quark &u, quark &d, quark &s, 
			    const double temper, thermo &th);
  
    /// Calculate equation of state as a function of density
    virtual int calc_temp_e(quark &u, quark &d, quark &s, 
			    const double temper, thermo &th);
  
    /// Return string denoting type ("quark_eos")
    virtual const char *type() { return "quark_eos"; }

  protected:

#ifndef DOXYGEN_INTERNAL
    
    friend class quark_eos_io;
    
#endif
    
  };

  template<> int io_tlate<quark_eos>::input
    (cinput *co, in_file_format *ins, quark_eos *sk);
  template<> int io_tlate<quark_eos>::output
    (coutput *co, out_file_format *outs, quark_eos *sk);
  template<> const char *io_tlate<quark_eos>::type();
  
  typedef io_tlate<quark_eos> quark_eos_io_type;
  
#ifndef DOXYGENP
}
#endif

#endif



