/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/fit_base.h>
#include <o2scl/fit_fix.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int fitfun(size_t n, ovector_view &p, double x, double &y, void *&pa) {
  y=p[0]*p[0]*x+p[1]*x+p[2];
  return 0;
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  double min, min2;
  ovector x(3);
  void *vp=0;
  
  size_t ndat=40;
  ovector xdat(ndat), ydat(ndat), yerr(ndat), par(3);
  omatrix covar(3,3);
  double chi2;
  for(size_t i=0;i<ndat;i++) {
    xdat[i]=((double)i)/50.0;
    ydat[i]=sin(xdat[i]);
    yerr[i]=0.01;
  }

  fit_fix_pars<void *,bool *> g;
  
  cout.setf(ios::scientific);
  
  fit_funct_fptr<void *> mf(fitfun);
  
  par[0]=1.0;
  par[1]=1.0;
  par[2]=1.0;
  int ret=g.fit(ndat,xdat,ydat,yerr,3,par,covar,chi2,vp,mf);
  cout << par << endl;

  bool fixa[3]={false,false,false};
  par[0]=1.0;
  par[1]=1.0;
  par[2]=1.0;
  bool *fixp=fixa;
  ret=g.fit_fix(ndat,xdat,ydat,yerr,3,par,fixp,covar,chi2,vp,mf);
  cout << par << endl;
  
  par[0]=1.6;
  par[1]=1.0;
  par[2]=1.0;
  fixa[0]=true;
  ret=g.fit_fix(ndat,xdat,ydat,yerr,3,par,fixp,covar,chi2,vp,mf);
  fixa[0]=false;
  cout << par << endl;

  par[0]=1.0;
  par[1]=-1.7;
  par[2]=1.0;
  fixa[1]=true;
  ret=g.fit_fix(ndat,xdat,ydat,yerr,3,par,fixp,covar,chi2,vp,mf);
  fixa[1]=false;
  cout << par << endl;

  par[0]=1.0;
  par[1]=1.0;
  par[2]=0.016;
  fixa[2]=true;
  ret=g.fit_fix(ndat,xdat,ydat,yerr,3,par,fixp,covar,chi2,vp,mf);
  fixa[2]=false;
  cout << par << endl;

  t.report();
  return 0;
}
 
