/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/funct.h>
#include <o2scl/cern_cauchy.h>

using namespace std;
using namespace o2scl;

double testfun(double tx, void *&pa);
double testfun2(double tx, void *&pa);
double testfun3(double tx, void *&pa);

funct_fptr_noerr<void *> tf(testfun);
funct_fptr_noerr<void *> tf2(testfun2);
funct_fptr_noerr<void *> tf3(testfun3);

double testfun(double tx, void *&pa) {
  return exp(-tx)/(tx-2.0);
}

double testfun2(double tx, void *&pa) {
  return log(tx+1.0)/(tx-1.0);
}

double testfun3(double tx, void *&pa) {
  return log(tx+1.0)/pow((exp(tx/10.0)-0.1),2.0);
}

int main(void) {
  double a=3.0, calc, exact, diff;
  cern_cauchy<void *,funct_fptr_noerr<void *> > cc;
  test_mgr t;
  t.set_output_level(2);
  void *vp=NULL;

  cout.setf(ios::scientific);
  cout.precision(10);
  
  cc.s=2.0;
  calc=cc.integ(tf,1.0,3.0,vp);
  t.test_rel(calc,-0.286166,1.0e-5,"cern_cauchy 1");
  
  cc.s=1.0;
  calc=cc.integ(tf2,0.0,2.0,vp);
  t.test_rel(calc,1.03065,1.0e-5,"cern_cauchy 2");

  cc.s=1.0;
  calc=cc.integ(tf3,0.0,2.0,vp);
  t.test_rel(calc,1.21569,1.0e-5,"cern_cauchy 3");
  
  t.report();
  return 0;
}

