/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/constants.h>
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qagi.h>

using namespace std;
using namespace o2scl;

int testfun2(double x, double &y, void *&pa) {
  y=exp(-x*x);
  return 0;
}

double f(double x, void *params) {
  double f=exp(-x*x);
  return f;
}
int main(void) {
  test_mgr t;
  t.set_output_level(2);
  gsl_inte_qagi<void *, funct_fptr<void *> > it;
  funct_fptr<void *> tf(&testfun2);

  double ans, exact;
  void *vp=NULL;

  cout.setf(ios::scientific);
  cout.precision(10);
  
  cout << "gsl_inte_qagi:" << endl;
  exact=sqrt(o2scl_const::pi);
  ans=it.integ(tf,0.0,0.0,vp);
  t.test_rel(ans,exact,1.0e-8,"qagi test");

  /// Compare with GSL and make sure we get the same result
  {
    gsl_integration_workspace *w=gsl_integration_workspace_alloc(1000);
    
    double result, error;
    double alpha = 0.01;
    
    gsl_function F;
    F.function = &f;
    F.params = &alpha;
    
    gsl_integration_qagi(&F,0,1e-7,1000,w,&result,&error); 
    
    t.test_rel(ans,result,1.0e-14,"GSL vs. O2scl");
    
    gsl_integration_workspace_free(w);
  }

  t.report();
  return 0;
}
