/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_INTE_QAGS_H
#define O2SCL_GSL_INTE_QAGS_H

#include <o2scl/inte.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qag.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief Integrate a function with a singularity (GSL)
   */
  template<class param_t, class func_t=funct<void *> > class gsl_inte_qags : 
  public gsl_inte_singular<param_t,func_t> {
    
    public:
      
    /** \brief Integrate function \c func from \c a to \c b.
     */
    virtual double integ(func_t &func, double a, double b, param_t &pa) {
      double res, err;
      integ_err(func,a,b,pa,res,err);
      this->interror=err;
      return res;
    }
      
    /** \brief Integrate function \c func from \c a to \c b and place
	the result in \c res and the error in \c err
    */
    virtual int integ_err(func_t &func, double a, double b, 
		  param_t &pa, double &res, double &err2) {
	
      double fv1[11], fv2[11];
	
      int status=qags(func,11,o2scl_inte_qag_coeffs::qk21_xgk,
		      o2scl_inte_qag_coeffs::qk21_wg,
		      o2scl_inte_qag_coeffs::qk21_wgk,
		      fv1,fv2,a,b,this->tolx,this->tolf,this->wkspace,
		      &res,&err2,pa);

      return status;
	
    }
            
           
  };
  
#ifndef DOXYGENP
}
#endif

#endif
