/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GIVENS_H
#define O2SCL_GIVENS_H

#include <o2scl/err_hnd.h>
#include <o2scl/permutation.h>
#include <o2scl/cblas.h>
#include <o2scl/vec_arith.h>

/** \file givens.h
    \brief File for Givens rotations
*/

/// Namespace for O2scl linear algebra function templates with operator[]
namespace o2scl_linalg {
  
  /// Desc
  void create_givens(const double a, const double b, double &c, 
                     double &s);
  
#define O2SCL_IX(V,i) V[i]
#define O2SCL_IX2(M,i,j) M[i][j]
#include <o2scl/givens_base.h>  
#undef O2SCL_IX
#undef O2SCL_IX2

}

/** 
    \brief Namespace for O2scl linear algebra function templates with 
    operator()
    
    This namespace contains an identical copy of all the functions given 
    in the \ref o2scl_cblas namespace, but perform array indexing
    with \c operator() rather than \c operator[]. See \ref o2scl_linalg
    for the function listing and documentation.
*/
namespace o2scl_linalg_paren {
  
  void create_givens(const double a, const double b, double &c, 
                     double &s);
  
#define O2SCL_IX(V,i) V(i)
#define O2SCL_IX2(M,i,j) M(i,j)
#include <o2scl/givens_base.h>  
#undef O2SCL_IX
#undef O2SCL_IX2
  
}

#endif
