/*
  -------------------------------------------------------------------

  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner

  This file is part of O2scl.

  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>

#include <gsl/gsl_linalg.h>

#include <o2scl/householder.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  {
    using namespace o2scl_cblas;
    using namespace o2scl_linalg;

    omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5);
    ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
    permutation gp1(5), op1(5), mp1(5);
    double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5];
    int sig, ret;
    for(size_t i=0;i<5;i++) {
      gv1[i]=cos(i);
      ov1[i]=cos(i);
      mv1[i]=cos(i);
      for(size_t j=0;j<5;j++) {
	gm1[i][j]=1.0+sin(i)+tan(j);
	om1[i][j]=1.0+sin(i)+tan(j);
	mm1[i][j]=1.0+sin(i)+tan(j);
	gm2[i][j]=1.0+sin(i)+tan(j);
	om2[i][j]=1.0+sin(i)+tan(j);
	mm2[i][j]=1.0+sin(i)+tan(j);
      }
    }
  
    gsl_linalg_householder_transform(&gv1);
    householder_transform<ovector>(5,ov1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h trans");

    gsl_linalg_householder_hm(1.5,&gv1,&gm1);
    householder_hm(5,5,1.5,ov1,om1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h hm 1");
    t.test_rel_mat(5,5,gm1,om1,1.0e-12,"h hm 2");
  
    for(size_t i=0;i<5;i++) {
      gv1[i]=cos(i);
      ov1[i]=cos(i);
      mv1[i]=cos(i);
      gv2[i]=cos(i);
      ov2[i]=cos(i);
      mv2[i]=cos(i);
    }
    gsl_linalg_householder_hv(1.5,&gv2,&gv1);
    householder_hv<ovector>(5,1.5,ov2,ov1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h hv 1");
    t.test_rel_arr(5,gv2,ov2,1.0e-12,"h hv 2");

  }
  {
    using namespace o2scl_cblas_paren;
    using namespace o2scl_linalg_paren;

    omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5);
    ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
    permutation gp1(5), op1(5), mp1(5);
    double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5];
    int sig, ret;
    for(size_t i=0;i<5;i++) {
      gv1[i]=cos(i);
      ov1[i]=cos(i);
      mv1[i]=cos(i);
      for(size_t j=0;j<5;j++) {
	gm1[i][j]=1.0+sin(i)+tan(j);
	om1[i][j]=1.0+sin(i)+tan(j);
	mm1[i][j]=1.0+sin(i)+tan(j);
	gm2[i][j]=1.0+sin(i)+tan(j);
	om2[i][j]=1.0+sin(i)+tan(j);
	mm2[i][j]=1.0+sin(i)+tan(j);
      }
    }
  
    gsl_linalg_householder_transform(&gv1);
    householder_transform<ovector>(5,ov1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h trans");

    gsl_linalg_householder_hm(1.5,&gv1,&gm1);
    householder_hm(5,5,1.5,ov1,om1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h hm 1");
    t.test_rel_mat(5,5,gm1,om1,1.0e-12,"h hm 2");
  
    for(size_t i=0;i<5;i++) {
      gv1[i]=cos(i);
      ov1[i]=cos(i);
      mv1[i]=cos(i);
      gv2[i]=cos(i);
      ov2[i]=cos(i);
      mv2[i]=cos(i);
    }
    gsl_linalg_householder_hv(1.5,&gv2,&gv1);
    householder_hv<ovector>(5,1.5,ov2,ov1);
    t.test_rel_arr(5,gv1,ov1,1.0e-12,"h hv 1");
    t.test_rel_arr(5,gv2,ov2,1.0e-12,"h hv 2");

  }

  t.report();
  return 0;
}

