/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_MCARLO_INTE_H
#define O2SCL_MCARLO_INTE_H

#include <iostream>
#include <o2scl/collection.h>
#include <o2scl/multi_inte.h>
#include <o2scl/rnga.h>
#include <o2scl/gsl_rnga.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Monte-Carlo integration [abstract base]
      
      This class provides the generic Monte Carlo parameters and the
      random number generator. The default type for the random number
      generator is a \ref gsl_rnga object. 
  */
  template<class param_t, class func_t, class rng_t=gsl_rnga, 
    class vec_t=ovector_view> class mcarlo_inte : 
    public multi_inte<param_t,func_t,vec_t> {

      public:
      
      mcarlo_inte() {
	n_points=1000;
      }

      virtual ~mcarlo_inte() {}
  
      /** 
	  \brief Number of integration points (default 1000)
      */
      int n_points;
    
      /// The random number generator
      rng_t def_rng;
  
      /// Return string denoting type ("mcarlo_inte")
      virtual const char *type() { return "mcarlo_inte"; }
  
    };

#ifndef DOXYGENP
}
#endif

#endif

