/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_mmin_conp.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

double minfun(size_t n, const ovector_view &x, void *&pa) {
  double ret;
  ret=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return(ret);
}

int minfund(size_t n, ovector_view &x, ovector_view &g, void *&pa) {
  double ret;
  g[0]=2.0*x[0];
  g[1]=2.0*(x[1]-2.0);
  return 0;
}

int minfun2(size_t n, const double x[2], double &y, void *&pa) {
  y=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return 0;
}

int minfund2(size_t n, double x[2], double g[2], void *&pa) {
  double ret;
  g[0]=2.0*x[0];
  g[1]=2.0*(x[1]-2.0);
  return 0;
}

double my_f(const gsl_vector *v, void *pa) {
  double x = gsl_vector_get(v,0);
  double y = gsl_vector_get(v,1);

  return x*x+(y-2.0)*(y-2.0)+3.0;
}

void my_df(const gsl_vector *v, void *params, gsl_vector *df) {
  double x = gsl_vector_get(v,0);
  double y = gsl_vector_get(v,1);
  
  gsl_vector_set(df,0,2.0*x);
  gsl_vector_set(df,1,2.0*(y-2.0));
}

void my_fdf(const gsl_vector *x, void *params, double *f, gsl_vector *df) {
  *f = my_f(x,params);
  my_df(x,params,df);
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  double min;
  ovector x(2);
  double xa[2];
  void *vp=0;
  int ret;
  gsl_mmin_conp<void *> g;
  gsl_mmin_conp_array<void *,2> g2;

  cout.setf(ios::scientific);

  multi_funct_fptr_noerr<void *> mf(minfun);
  grad_funct_fptr<void *> mfd(minfund);
  multi_vfunct_fptr<void *,2> mf2(minfun2);
  grad_vfunct_fptr<void *,2> mfd2(minfund2);

  // Normal version 
  
  x[0]=1.0;
  x[1]=1.0;
  ret=g.mmin(2,x,min,vp,mf);
  cout << ret << " " << min << " " << x[0] << " " << x[1] << endl;
  t.test_gen(ret==0,"gsl_mmin_conp 0");
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_conp 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_conp 2");
  cout << endl;

  // Array version with gradient

  xa[0]=1.0;
  xa[1]=1.0;
  ret=g2.mmin(2,xa,min,vp,mf2);
  cout << ret << " " << min << " " << xa[0] << " " << xa[1] << endl;
  t.test_gen(ret==0,"gsl_mmin_conp 0");
  t.test_abs(xa[0],0.0,1.0e-4,"gsl_mmin_conp 1");
  t.test_rel(xa[1],2.0,2.0e-4,"gsl_mmin_conp 2");
  cout << endl;

  // Array version with gradient

  xa[0]=1.0;
  xa[1]=1.0;
  ret=g2.mmin_de(2,xa,min,vp,mf2,mfd2);
  cout << ret << " " << min << " " << xa[0] << " " << xa[1] << endl;
  t.test_gen(ret==0,"gsl_mmin_conp 0");
  t.test_abs(xa[0],0.0,1.0e-4,"gsl_mmin_conp 1");
  t.test_rel(xa[1],2.0,2.0e-4,"gsl_mmin_conp 2");
  cout << endl;

  // Normal version with gradient

  x[0]=1.0;
  x[1]=1.0;
  g.step_size=0.01;
  g.lmin_tol=1.0e-4;
  g.verbose=1;
  ret=g.mmin_de(2,x,min,vp,mf,mfd);
  cout << ret << " " << min << " " << x[0] << " " << x[1] << endl;
  t.test_gen(ret==0,"gsl_mmin_conp 0");
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_conp 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_conp 2");
  cout << endl;

  // GSL version
  {
    size_t iter = 0;
    int status;
    
    const gsl_multimin_fdfminimizer_type *T;
    gsl_multimin_fdfminimizer *s;
    
    gsl_vector *gx;
    gsl_multimin_function_fdf my_func;
    
    my_func.f = &my_f;
    my_func.df = &my_df;
    my_func.fdf = &my_fdf;
    my_func.n = 2;
    my_func.params = 0;
    
    gx = gsl_vector_alloc (2);
    gsl_vector_set(gx,0,1.0);
    gsl_vector_set(gx,1,1.0);
    T = gsl_multimin_fdfminimizer_conjugate_pr;
    s = gsl_multimin_fdfminimizer_alloc (T, 2);
    
    gsl_multimin_fdfminimizer_set(s,&my_func,gx,0.01,1e-4);
    
    do {
      iter++;
      status = gsl_multimin_fdfminimizer_iterate(s);
      cout << "Iter: " << iter << " " << gsl_vector_get(s->x,0) << " "
	   << gsl_vector_get(s->x,1) << " " 
	   << gsl_vector_get(s->gradient,0) << " "
	   << gsl_vector_get(s->gradient,1) << " "
	   << endl;
      
      if (status)
	break;
      
      status = gsl_multimin_test_gradient(s->gradient,1e-4);
      
    } while (status == GSL_CONTINUE && iter < 100);

    cout << "Found at : " << gsl_vector_get(s->x,0) << " "
	 << gsl_vector_get(s->x,1) << endl;
    
    gsl_multimin_fdfminimizer_free (s);
    gsl_vector_free(gx);
  }

  t.report();
  return 0;
}
 
