/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/quad_intp.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  cout.setf(ios::scientific);

  ovector x(8), y(8), d(8), *dp=new ovector[1];
  dp[0].allocate(8);
  
  quad_intp<ovector,ovector *> pi;
  ovector res(1);

  x[0]=0.03; y[0]=0.01; 
  x[1]=0.03; y[1]=1.01; 
  x[2]=1.04; y[2]=1.02; 
  x[3]=1.04; y[3]=0.02; 
  x[4]=0.81; y[4]=0.23; 
  x[5]=0.21; y[5]=0.83; 
  x[6]=0.82; y[6]=0.84; 
  x[7]=0.22; y[7]=0.24; 
  
  for(size_t i=0;i<8;i++) {
    dp[0][i]=1.0-pow(x[i]-0.5,2.0)-pow(y[i]-0.5,2.0);
  }

  pi.set_data(8,x,y,1,dp);

  int r=pi.interp(0.04,0.01,res);
  cout << r << " " << res << " " 
       << 1.0-pow(0.04-0.5,2.0)-pow(0.01-0.5,2.0) << endl;
  
  for(double yy=0.0;yy<=1.0;yy+=0.1) {
    int ret=pi.interp(0.5,yy,res);
    t.test_gen(ret==0,"err value");
    cout << res[0] << " " << 1.0-pow(yy-0.5,2.0) << endl;
    //t.test_rel(res[0],1.0-pow(yy-0.5,2.0),0.33,"quad_intp");
  }

  t.report();
  return 0;
}
