/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/twod_intp.h>

using namespace std;
using namespace o2scl;

double f(double x, double y) {
  return pow(sin(0.1*x+0.3*y),2.0);
}

double fix(double x0, double x1, double y) {
  return (x1-x0)/2.0+(sin(2.0*(0.1*x0+0.3*y))-sin(2.0*(0.1*x1+0.3*y)))/0.4;
}

double fiy(double x, double y0, double y1) {
  return (y1-y0)/2.0+(sin(2.0*(0.1*x+0.3*y0))-sin(2.0*(0.1*x+0.3*y1)))/1.2;
}

double fx(double x, double y) {
  return 2.0*sin(0.1*x+0.3*y)*0.1*cos(0.1*x+0.3*y);
}

double fy(double x, double y) {
  return 2.0*sin(0.1*x+0.3*y)*0.3*cos(0.1*x+0.3*y);
}

double f2x(double x, double y) {
  return 2.0*0.1*0.1*(pow(cos(0.1*x+0.3*y),2.0)-
		      pow(sin(0.1*x+0.3*y),2.0));
}

double f2y(double x, double y) {
  return 2.0*0.3*0.3*(pow(cos(0.1*x+0.3*y),2.0)-
		      pow(sin(0.1*x+0.3*y),2.0));
}

double fxy(double x, double y) {
  return 2.0*0.1*0.3*(pow(cos(0.1*x+0.3*y),2.0)-
		      pow(sin(0.1*x+0.3*y),2.0));
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);

  int i,j;
  twod_intp ti;
  double x0, x1, y0, y1;
  double tol=0.05;
  double tol2=0.4;

  {
    // N=M
    
    size_t N=40;
    size_t M=40;
    ovector x2(N), y2(M);
    omatrix data2(M,N);
    for(size_t ii=0;ii<N;ii++) {
      x2[ii]=((double)ii)/10.0;
    }
    for(size_t ii=0;ii<M;ii++) {
      y2[ii]=((double)ii)/20.0;
    }
    for(size_t ii=0;ii<N;ii++) {
      for(size_t jj=0;jj<M;jj++) {
	data2[jj][ii]=f(x2[ii],y2[jj]);
      }
    }
  
    // x-first

    x0=2.42;
    x1=3.22;
    y0=1.41;
    y1=0.62;

    ti.set_data(40,40,x2,y2,data2,true);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintp x 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintp x 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintp x 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintp x 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintp x 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintp x 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintp x 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintp x 1 diy");
      }
    }

    // y-first

    x0=2.42;
    y0=1.41;
    y0=1.41;
    y1=0.62;

    ti.set_data(40,40,x2,y2,data2,false);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp y 1 i");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintp y 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintp y 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintp y 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintp y 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintp y 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintp y 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintp y 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintp y 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintp y 1 diy");
      }
    }

  }

  {
    // M>N
    
    size_t N=40;
    size_t M=50;
    ovector x2(N), y2(M);
    omatrix data2(M,N);
    for(size_t ii=0;ii<N;ii++) {
      x2[ii]=((double)ii)/10.0;
    }
    for(size_t ii=0;ii<M;ii++) {
      y2[ii]=((double)ii)/20.0;
    }
    for(size_t ii=0;ii<N;ii++) {
      for(size_t jj=0;jj<M;jj++) {
	data2[jj][ii]=f(x2[ii],y2[jj]);
      }
    }
  
    // x-first

    x0=2.42;
    x1=3.22;
    y0=1.41;
    y1=0.62;

    ti.set_data(N,M,x2,y2,data2,true);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpM x 1 dx");
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpM x 1 dx");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintpM x 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintpM x 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintpM x 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintpM x 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintpM x 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintpM x 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintpM x 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintpM x 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintpM x 1 diy");
      }
    }

    // y-first

    x0=2.42;
    y0=1.41;
    y0=1.41;
    y1=0.62;

    ti.set_data(N,M,x2,y2,data2,false);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpM y 1 i");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintpM y 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintpM y 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintpM y 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintpM y 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintpM y 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintpM y 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintpM y 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintpM y 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintpM y 1 diy");
      }
    }

  }

  {
    // N>M
    
    size_t N=50;
    size_t M=40;
    ovector x2(N), y2(M);
    omatrix data2(M,N);
    for(size_t ii=0;ii<N;ii++) {
      x2[ii]=((double)ii)/10.0;
    }
    for(size_t ii=0;ii<M;ii++) {
      y2[ii]=((double)ii)/20.0;
    }
    for(size_t ii=0;ii<N;ii++) {
      for(size_t jj=0;jj<M;jj++) {
	data2[jj][ii]=f(x2[ii],y2[jj]);
      }
    }
  
    // x-first

    x0=2.42;
    x1=3.22;
    y0=1.41;
    y1=0.62;

    ti.set_data(N,M,x2,y2,data2,true);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpN x 1 dx");
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpN x 1 dx");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintpN x 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintpN x 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintpN x 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintpN x 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintpN x 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintpN x 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintpN x 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintpN x 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintpN x 1 diy");
      }
    }

    // y-first

    x0=2.42;
    y0=1.41;
    y0=1.41;
    y1=0.62;

    ti.set_data(N,M,x2,y2,data2,false);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintpN y 1 i");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintpN y 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintpN y 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),tol,"2dintpN y 1 dyx");
      t.test_rel(ti.deriv2_x(x0,y0),f2x(x0,y0),tol2,"2dintpN y 1 d2x");
      t.test_rel(ti.deriv2_y(x0,y0),f2y(x0,y0),tol2,"2dintpN y 1 d2y");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintpN y 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintpN y 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintpN y 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintpN y 1 diy");
      }
    }

  }

  {
    // N=M with linear interpolation
    
    size_t N=40;
    size_t M=40;
    ovector x2(N), y2(M);
    omatrix data2(M,N);
    for(size_t ii=0;ii<N;ii++) {
      x2[ii]=((double)ii)/10.0;
    }
    for(size_t ii=0;ii<M;ii++) {
      y2[ii]=((double)ii)/20.0;
    }
    for(size_t ii=0;ii<N;ii++) {
      for(size_t jj=0;jj<M;jj++) {
	data2[jj][ii]=f(x2[ii],y2[jj]);
      }
    }
  
    // x-first

    x0=2.42;
    x1=3.22;
    y0=1.41;
    y1=0.62;

    linear_interp<ovector_view> bi[41];
    linear_interp<ovector_const_subvector> bi2[41];
    ti.unset_data();
    ti.set_interp(40,bi,bi2,bi[40],bi2[40]);
    ti.set_data(40,40,x2,y2,data2,true);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintp x 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintp x 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),1.5,"2dintp x 1 dyx");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintp x 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintp x 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintp x 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintp x 1 diy");
      }
    }

    // y-first

    x0=2.42;
    y0=1.41;
    y0=1.41;
    y1=0.62;

    ti.set_data(40,40,x2,y2,data2,false);

    for(size_t im=0;im<10;im++) {
      x0*=sqrt(5.0);
      x1*=sqrt(5.0);
      y0*=sqrt(7.0);
      y1*=sqrt(7.0);
      while (x0>x2[N-1]) x0/=2.0;
      while (x1>x2[N-1]) x1/=2.0;
      while (y0>y2[M-1]) y0/=2.0;
      while (y1>y2[M-1]) y1/=2.0;
      t.test_rel(ti.interp(x0,y0),f(x0,y0),tol,"2dintp y 1 i");
      t.test_rel(ti.deriv_x(x0,y0),fx(x0,y0),tol,"2dintp y 1 dx");
      t.test_rel(ti.deriv_y(x0,y0),fy(x0,y0),tol,"2dintp y 1 dy");
      t.test_rel(ti.deriv_xy(x0,y0),fxy(x0,y0),1.5,"2dintp y 1 dyx");
      if (x0<x1) {
	t.test_rel(ti.integ_x(x0,x1,y0),fix(x0,x1,y0),tol2,"2dintp y 1 dix");
      } else {
	t.test_rel(ti.integ_x(x1,x0,y0),fix(x1,x0,y0),tol2,"2dintp y 1 dix");
      }
      if (y0<y1) {
	t.test_rel(ti.integ_y(x0,y0,y1),fiy(x0,y0,y1),tol2,"2dintp y 1 diy");
      } else {
	t.test_rel(ti.integ_y(x0,y1,y0),fiy(x0,y1,y0),tol2,"2dintp y 1 diy");
      }
    }

  }


  t.report();
  return 0;
}

