/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/eff_fermion.h>
#include <o2scl/rel_fermion.h>
#include <o2scl/classical.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {
  double tmp1, tmp2, tmp3, tmp4, tmp5, T;
  int ret;
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  eff_fermion e(1.0,2.0);
  t.test_rel(e.m,1.0,1.0e-6,"mass inheritance");
  e.non_interacting=true;

  fermion ferm(1.0,2.0);
  double den;
  
  cout << "----------------------------------------------------" << endl;
  cout << "Four combinations of inc_rest_mass & non_interacting" << endl;
  cout << "----------------------------------------------------" << endl;
  cout << endl;

  for(den=1.0e-6;den<=1.01e2;den*=1.0e2) {

    cout << "density: " << den << endl;

    // -----------------------------------------------------------------
    // Test calc_density() and pair_density() with inc_rest_mass is 
    // both true and false, and non_interacting is both true and false
    // -----------------------------------------------------------------
  
    eff_fermion xe(0.511/hc_mev_fm,2.0);
    double temper=10.0/hc_mev_fm, d1, d2, mu;
    double t1, t2, t3, t4;
    int rev, rev2;

    // -----------------------------------------------------------------

    xe.non_interacting=false;
    cout << "Interacting: " << endl;
    cout << endl;
    
    xe.inc_rest_mass=true;
    cout << "Include rest mass." << endl;
    
    xe.n=den;
    xe.nu=xe.m;
    rev=xe.calc_density(temper);
    cout << xe.nu << " " << rev << " ";
    t1=xe.nu;
    rev2=xe.calc_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den1a");

    xe.n=den;
    xe.nu=xe.m;
    rev=xe.pair_density(temper);
    mu=xe.nu;
    cout << xe.nu << " " << rev << " ";
    t2=xe.nu;
    rev2=xe.pair_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den2a");
    
    xe.nu=mu;
    rev=xe.calc_mu(temper);
    d1=xe.n;
    if (xe.inc_rest_mass) {
      xe.nu=-mu;
    } else {
      xe.nu=-mu-2.0*xe.m;
    }
    rev2=xe.calc_mu(temper);
    d2=xe.n;
    cout << rev << " " << rev2 << " "
	 << den << " " << d1 << " " << d2 << " " << d1-d2 << endl;
    t3=d1;
    t4=d2;
    t.test_rel(den,d1-d2,1.0e-6,"den3a");
    cout << endl;

    // -----------------------------------------------------------------

    xe.inc_rest_mass=false;
    cout << "Remove rest mass." << endl;
    
    xe.n=den;
    xe.nu=0.0;
    rev=xe.calc_density(temper);
    cout << xe.nu+xe.m << " " << rev << " ";
    t.test_rel(xe.nu+xe.m,t1,1.0e-6,"nu1b");
    rev2=xe.calc_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den1b");

    xe.n=den;
    xe.nu=0.0;
    rev=xe.pair_density(temper);
    mu=xe.nu;
    cout << xe.nu+xe.m << " " << rev << " ";
    t.test_rel(xe.nu+xe.m,t2,1.0e-6,"nu2b");
    rev2=xe.pair_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den2b");
    
    xe.nu=mu;
    rev=xe.calc_mu(temper);
    d1=xe.n;
    xe.nu=-mu-2.0*xe.m;
    rev2=xe.calc_mu(temper);
    d2=xe.n;
    cout << rev << " " << rev2 << " "
	 << den << " " << d1 << " " << d2 << " " << d1-d2 << endl;
    t.test_rel(den,d1-d2,1.0e-6,"den3b");
    t.test_rel(t3,d1,1.0e-6,"den4b");
    t.test_rel(t4,d2,1.0e-6,"den5b");
    cout << endl;

    // -----------------------------------------------------------------

    xe.non_interacting=true;
    cout << "Non-interacting: " << endl;
    cout << endl;

    xe.inc_rest_mass=true;
    cout << "Include rest mass." << endl;
    
    xe.n=den;
    xe.mu=0.0;
    rev=xe.calc_density(temper);
    cout << xe.mu << " " << rev << " ";
    t.test_rel(xe.nu,t1,1.0e-6,"nu1c");
    rev2=xe.calc_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den1c");
    
    xe.n=den;
    xe.mu=0.0;
    rev=xe.pair_density(temper);
    mu=xe.mu;
    cout << xe.mu << " " << rev << " ";
    t.test_rel(xe.nu,t2,1.0e-6,"nu2c");
    rev2=xe.pair_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den2c");
    
    xe.mu=mu;
    rev=xe.calc_mu(temper);
    d1=xe.n;
    xe.mu=-mu;
    rev2=xe.calc_mu(temper);
    d2=xe.n;
    cout << rev << " " << rev2 << " "
	 << den << " " << d1 << " " << d2 << " " << d1-d2 << endl;
    t.test_rel(den,d1-d2,1.0e-6,"den3c");
    t.test_rel(t3,d1,1.0e-6,"den4c");
    t.test_rel(t4,d2,1.0e-6,"den5c");
    cout << endl;

    // -----------------------------------------------------------------

    xe.inc_rest_mass=false;
    cout << "Remove rest mass." << endl;
    
    xe.n=den;
    xe.mu=0.0;
    rev=xe.calc_density(temper);
    cout << xe.mu+xe.m << " " << rev << " ";
    t.test_rel(xe.nu+xe.m,t1,1.0e-6,"nu1d");
    rev2=xe.calc_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den1d");
    
    xe.n=den;
    xe.mu=0.0;
    rev=xe.pair_density(temper);
    mu=xe.mu;
    cout << xe.mu+xe.m << " " << rev << " ";
    t.test_rel(xe.nu+xe.m,t2,1.0e-6,"nu2d");
    rev2=xe.pair_mu(temper);
    cout << xe.n << " " << den << " " << rev2 << endl;
    t.test_rel(xe.n,den,1.0e-6,"den2d");
    
    xe.mu=mu;
    rev=xe.calc_mu(temper);
    d1=xe.n;
    xe.mu=-mu-2.0*xe.m;
    rev2=xe.calc_mu(temper);
    d2=xe.n;
    cout << rev << " " << rev2 << " "
	 << den << " " << d1 << " " << d2 << " " << d1-d2 << endl;
    t.test_rel(den,d1-d2,1.0e-6,"den3d");
    t.test_rel(t3,d1,1.0e-6,"den4d");
    t.test_rel(t4,d2,1.0e-6,"den5d");
    cout << endl;

  }

  cout << "----------------------------------------------------" << endl;
  cout << "eff_fermion tests:" << endl;
  cout << "----------------------------------------------------" << endl;
  cout << endl;

  cout << "degenerate extremely relativistic (mu>>m, mu>T) "
       << "(m=0.01, mu=1.0, T=0.1)" << endl;
  e.m=0.01;
  e.mu=1.0;
  T=0.1;
  cout << "calc_mu(T) vs. massless_calc_mu(T)" << endl;
  ret=e.calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl; 
  t.test_gen(ret==0,"eff_fermion return value 1");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.massless_calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 1b");
  t.test_rel(tmp1,e.n,5.0e-4,"eff_fermion density 1");
  t.test_rel(tmp2,e.mu,5.0e-4,"eff_fermion chem. pot. 1");
  t.test_rel(tmp3,e.ed,5.0e-4,"eff_fermion energy 1");
  t.test_rel(tmp4,e.pr,5.0e-4,"eff_fermion pressure 1");
  t.test_rel(tmp5,e.en,5.0e-4,"eff_fermion entropy 1");

  cout << "calc_density(T) vs. massless_calc_density(T)" << endl;
  e.m=0.01;
  e.mu=1.0;
  T=0.1;
  ret=e.calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 2");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.massless_calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 2b");

  t.test_rel(tmp1,e.n,5.0e-4,"eff_fermion density 2");
  t.test_rel(tmp2,e.mu,5.0e-4,"eff_fermion chem. pot. 2");
  t.test_rel(tmp3,e.ed,5.0e-4,"eff_fermion energy 2");
  t.test_rel(tmp4,e.pr,5.0e-4,"eff_fermion pressure 2");
  t.test_rel(tmp5,e.en,5.0e-4,"eff_fermion entropy 2");
  cout << endl;

  cout << "extremely degenerate relativistic (mu>m, mu>>T) "
       << "(m=0.1, mu=1.0, T=0.01)" << endl;
  e.m=0.1;
  e.mu=1.0;
  T=0.01;
  cout << "calc_mu(T) vs. calc_mu(T=0)" << endl;
  ret=e.calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 3");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.calc_mu_zerot();
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 3b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 3");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 3");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 3");
  t.test_rel(tmp4,e.pr,5.0e-3,"eff_fermion pressure 3");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 3");

  cout << "calc_density(T) vs. calc_density(T=0)" << endl;
  e.m=0.1;
  e.mu=1.0;
  T=0.01;
  ret=e.calc_mu(T);
  ret=e.calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 4");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.calc_density_zerot();
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 4b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 4");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 4");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 4");
  t.test_rel(tmp4,e.pr,5.0e-3,"eff_fermion pressure 4");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 4");
  cout << endl;

  cout << "degenerate non-relativistic ((mu-m=kf^2/2/m)<<m, T<<mu) "
       << "(m=1.0, mu=1.1, T=0.01)" << endl;
  e.m=1.0;
  e.mu=1.1;
  T=0.01;
  cout << "calc_mu(T) vs. calc_mu(T=0)" << endl;
  ret=e.calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 5");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.calc_mu_zerot();
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 5b");
  t.test_rel(tmp1,e.n,3.0e-2,"eff_fermion density 5");
  t.test_rel(tmp2,e.mu,3.0e-2,"eff_fermion chem. pot. 5");
  t.test_rel(tmp3,e.ed,3.0e-2,"eff_fermion energy 5");
  t.test_rel(tmp4,e.pr,8.0e-2,"eff_fermion pressure 5");
  t.test_rel(tmp5,e.en,3.0e-2,"eff_fermion entropy 5");

  cout << "calc_density(T) vs. calc_density(T=0)" << endl;
  e.m=1.0;
  e.mu=1.1;
  T=0.01;
  ret=e.calc_mu(T);
  ret=e.calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 6");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.calc_density_zerot();
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 6b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 6");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 6");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 6");
  t.test_rel(tmp4,e.pr,5.0e-2,"eff_fermion pressure 6");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 6");
  cout << endl;

  cout << "non-degenerate extremely relativistic (m<<mu<<T) "
       << "(m=0.01, mu=0.1, T=1.0)" << endl;
  e.m=0.01;
  e.mu=0.1;
  T=1.0;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=e.calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 7");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  e.mu*=2.0;
  ret=e.calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 7b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 7");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 7");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 7");
  t.test_rel(tmp4,e.pr,5.0e-3,"eff_fermion pressure 7");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 7");

  cout << "pair_mu(T) vs. massless_pair_mu(T) vs. pair_density(T) vs. " 
       << endl;
  cout << "massless_pair_density(T)" << endl;
  e.m=0.01;
  e.mu=0.1;
  T=1.0;
  ret=e.pair_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 8");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.massless_pair_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 8b");
  t.test_rel(tmp1,e.n,1.0e-2,"eff_fermion density 8");
  t.test_rel(tmp2,e.mu,1.0e-2,"eff_fermion chem. pot. 8");
  t.test_rel(tmp3,e.ed,1.0e-2,"eff_fermion energy 8");
  t.test_rel(tmp4,e.pr,1.0e-2,"eff_fermion pressure 8");
  t.test_rel(tmp5,e.en,1.0e-2,"eff_fermion entropy 8");

  e.m=0.01;
  e.mu=0.1;
  T=1.0;
  ret=e.pair_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 9");
  t.test_rel(tmp1,e.n,5.0e-4,"eff_fermion density 9c");
  t.test_rel(tmp2,e.mu,5.0e-4,"eff_fermion chem. pot. 9c");
  t.test_rel(tmp3,e.ed,5.0e-4,"eff_fermion energy 9c");
  t.test_rel(tmp4,e.pr,5.0e-4,"eff_fermion pressure 9c");
  t.test_rel(tmp5,e.en,5.0e-4,"eff_fermion entropy 9c");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  ret=e.massless_pair_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 9b");
  t.test_rel(tmp1,e.n,1.0e-2,"eff_fermion density 9");
  t.test_rel(tmp2,e.mu,1.0e-2,"eff_fermion chem. pot. 9");
  t.test_rel(tmp3,e.ed,1.0e-2,"eff_fermion energy 9");
  t.test_rel(tmp4,e.pr,1.0e-2,"eff_fermion pressure 9");
  t.test_rel(tmp5,e.en,1.0e-2,"eff_fermion entropy 9");
  cout << endl;

  cout << "non-degenerate non-relativistic (mu<<m, mu<<T) "
       << "(m=0.1, mu=0.01, T=1.0)" << endl;
  e.m=0.1;
  e.mu=0.01;
  T=1.0;
  cout << "calc_mu(T) vs. calc_density(T)" << endl;
  ret=e.calc_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl; 
  t.test_gen(ret==0,"eff_fermion return value 10");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  e.mu*=2.0;
  ret=e.calc_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 10b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 10");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 10");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 10");
  t.test_rel(tmp4,e.pr,5.0e-3,"eff_fermion pressure 10");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 10");

  cout << "pair_mu(T) vs. pair_density(T)" << endl;
  e.m=0.1;
  e.mu=0.01;
  T=1.0;
  ret=e.pair_mu(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 11");
  tmp1=e.n;
  tmp2=e.mu;
  tmp3=e.ed;
  tmp4=e.pr;
  tmp5=e.en;
  e.mu*=2.0;
  ret=e.pair_density(T);
  cout << ret << " " << e.n << " " << e.mu << " " << e.ed << " " 
       << e.pr << " " << e.en << endl;
  t.test_gen(ret==0,"eff_fermion return value 11b");
  t.test_rel(tmp1,e.n,5.0e-3,"eff_fermion density 11");
  t.test_rel(tmp2,e.mu,5.0e-3,"eff_fermion chem. pot. 11");
  t.test_rel(tmp3,e.ed,5.0e-3,"eff_fermion energy 11");
  t.test_rel(tmp4,e.pr,5.0e-3,"eff_fermion pressure 11");
  t.test_rel(tmp5,e.en,5.0e-3,"eff_fermion entropy 11");
  cout << endl;
  
  // -----------------------------------------------------------------
  // This tests the results of eff_fermion at the fitting points used
  // in Johns, Ellis, and Lattimer and compares them to the results
  // from the rel_fermion class.
  
  cout << "----------------------------------------------------" << endl;
  cout << "eff_fermion vs. rel_fermion:" << endl;
  cout << "----------------------------------------------------" << endl;
  cout << endl;

  rel_fermion e3(1.0,2.0);

  cout.precision(4);
  int ret1, ret2;
  double logf, logg, psi, sqt, a=0.42, f, g;

  double maxdev=0.0, maxf=0.0, maxg=0.0;
  for(logf=-10.61;logf<=8.0;logf+=2.0) {
    for(logg=-7.0;logg<=5.01;logg+=2.0) {

      f=pow(10.0,logf);
      g=pow(10.0,logg);
      T=1.0;
      e.m=T/g*sqrt(1.0+f);
      sqt=sqrt(1.0+f/a);
      psi=2.0*sqt+log((sqt-1.0)/(sqt+1.0));
      e.mu=psi*T+e.m;
      e3.m=e.m;
      e3.mu=e.mu;
      ret1=e.calc_mu(T);
      ret2=e3.calc_mu(T);
      cout << ret1 << " " << ret2 << " ";
      cout.setf(ios::showpos);
      cout << logf << " " << logg << " "; 
      cout.unsetf(ios::showpos);
      cout << e.pr << " " << e3.pr << " " << e3.unc.pr << " ";
      t.test_gen(ret1==0,"ret value");
      t.test_rel(e.pr,e3.pr,1.0e-2,"eq. pressure from mu");
      if (fabs(e.pr-e3.pr)/e.pr>maxdev) {
	maxdev=fabs(e.pr-e3.pr)/e.pr;
	maxf=logf;
	maxg=logg;
      }
      ret2=e.calc_density(T);
      cout << ret2 << " " << e.pr << endl;
      t.test_gen(ret2==0,"ret value");
      t.test_rel(e.pr,e3.pr,1.0e-2,"eq. pressure from density");

    }

    if (logf<-10.0) logf+=3.61;
    if (logf>4.0) logf+=0.82;
  }
  cout << "Maximum deviation: " << maxdev << " at log(f): " << maxf 
       << " log(g): " << maxg << endl;
  cout << endl;

  {
    /// This code shows that classical::pair_mu() is incorrect
    eff_fermion ext(1.0,2.0);
    ext.mu=-1.0;
    ext.calc_mu(0.02);
    cout << ext.ed << endl;
    ext.pair_mu(0.02);
    cout << ext.ed << endl;
    classical ext2(1.0,2.0);
    ext2.mu=-1.0;
    ext2.calc_mu(0.02);
    cout << ext2.ed << endl;
    ext2.pair_mu(0.02);
    cout << ext2.ed << endl;
  }

  {
    // Show that the free energy decreases as a function of increasing
    // temperature at a particular fixed density
    eff_fermion ext(1.0,2.0);
    ext.n=0.01;
    ext.mu=ext.m;
    ext.calc_density(0.02);
    cout << ext.mu << " " << ext.ed << " " << ext.ed-0.02*ext.en << endl;
    ext.calc_density_zerot();
    cout << ext.mu << " " << ext.ed << " " << ext.ed << endl;
    
    rel_fermion ext2(1.0,2.0);
    ext2.n=0.01;
    ext2.calc_density(0.02);
    cout << ext2.mu << " " << ext2.ed << " " << ext2.ed-0.02*ext2.en << endl;
    ext2.calc_density_zerot();
    cout << ext2.mu << " " << ext2.ed << " " << ext2.ed << endl;
  }

  t.report();
  return 0;
}
  

