/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_QUARK_H
#define O2SCL_QUARK_H

#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <o2scl/constants.h>
#include <o2scl/mroot.h>
#include <o2scl/inte.h>
#include <o2scl/fermion.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Quark base

  */
  class quark : virtual public fermion {
  public:

    /// Contribution to the bag constant
    double B;

    /// Quark condensate
    double qq;

    /// Create a quark with mass \c m and degeneracy \c g
    quark(double mass=0.0, double dof=0.0);

    virtual int calc_mu(const double temper);
    virtual int calc_density(const double temper);
    virtual int pair_mu(const double temper);
    virtual int pair_density(const double temper);

    /// Return string denoting type ("quark")
    virtual const char *type() { return "quark"; }

#ifndef DOXYGENP  
    
    friend class io_tlate<quark>;

  protected:

#endif
  
  };

  template<> int io_tlate<quark>::input
    (cinput *co, in_file_format *ins, quark *q);
  template<> int io_tlate<quark>::output
    (coutput *co, out_file_format *outs, quark *q);
  template<> const char *io_tlate<quark>::type();
  
  typedef io_tlate<quark> quark_io_type;

#ifndef DOXYGENP
}
#endif

#endif
