/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <cmath>
#include <o2scl/test_mgr.h>
#include <o2scl/mm_funct.h>
#include <o2scl/cern_mroot.h>

int gfn(size_t nv, const o2scl::ovector_view &x, 
	o2scl::ovector_view &y, void *&pa) {
  y[0]=sin(x[1]-0.2);
  y[1]=sin(x[0]-0.25);
  return 0;
}

int gfn2(size_t nv, const double x[2], double y[2], void *&pa) {
  y[0]=sin(x[1]-0.2);
  y[1]=sin(x[0]-0.25);
  return 0;
}

class cl {
public:
  int mfn(size_t nv, const o2scl::ovector_view &x, 
	  o2scl::ovector_view &y, void *&pa) {
    y[0]=sin(x[1]-0.2);
    y[1]=sin(x[0]-0.25);
    return 0;
  }
  
  // Placeholder jacobian (ignored by cern_mroot)
  int mfnd(size_t nv, o2scl::ovector_view &x, o2scl::ovector_view &y,
	   double **&j, void *&pa) {
    return 0;
  }
  
  int mfn2(size_t nv, const double x[2], double y[2], void *&pa) {
    y[0]=sin(x[1]-0.2);
    y[1]=sin(x[0]-0.25);
    return 0;
  }
};

using namespace std;
using namespace o2scl;

int main(void) {
  cl acl;
  ovector x(2);
  double xa[2];
  int i;
  void *vp=NULL;
  size_t tmp;
  //  int N=700;
  int N=1;
  int t1=0, t2=0, t3=0, t4=0, t5=0;
  test_mgr t;

  t.set_output_level(1);

  for(int kk=0;kk<1;kk++) {

    // 1 - Non-templated access through a funct object 
    mm_funct_mfptr<cl,void *,ovector_view> fmf(&acl,&cl::mfn);
    cern_mroot<void *,mm_funct_mfptr<cl,void *,ovector_view>,
      ovector_view,ovector,ovector_alloc,
      jac_funct_mfptr<cl,void *,ovector_view,double **> > cr1;
      tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	cr1.msolve(2,x,vp,fmf);
      }
    }
    t1+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"1a");
    t.test_rel(x[1],0.2,1.0e-6,"1b");

    // Show that msolve_de() works
    jac_funct_mfptr<cl,void *,ovector_view,double **> fmfd(&acl,&cl::mfnd);
    x[0]=0.5;
    x[1]=0.5;
    cr1.msolve_de(2,x,vp,fmf,fmfd);
    t.test_rel(x[0],0.25,1.0e-6,"1c");
    t.test_rel(x[1],0.2,1.0e-6,"1d");

    // 2 - Non-templated access through a funct object 
    mm_vfunct_mfptr<cl,void *,2> fmf2(&acl,&cl::mfn2);
    cern_mroot<void *,mm_vfunct_mfptr<cl,void *,2>,double[2],double[2],
      array_alloc<double[2]> > cr2;
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	xa[0]=0.5;
	xa[1]=0.5;
	cr2.msolve(2,xa,vp,fmf2);
      }
    }
    t2+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"1a");
    t.test_rel(x[1],0.2,1.0e-6,"1b");
    
    // 4 - Templated access through a global function pointer
    typedef int (*gfnt)(size_t, const ovector_view &, ovector_view &, 
			void *&);
    cern_mroot<void *,gfnt,ovector_view,ovector,ovector_alloc > cr4;
    gfnt gfnv=&gfn;
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	x[0]=0.5;
	x[1]=0.5;
	cr4.msolve(2,x,vp,gfnv);
      }
    }
    t4+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"5a");
    t.test_rel(x[1],0.2,1.0e-6,"5b");

    // 5 - Templated access through a global function pointer
    typedef int (*gfnt2)(size_t, const double[2], double[2], void *&);
    cern_mroot<void *,gfnt2,double[2],double[2],array_alloc<double[2] > > cr42;
    gfnt2 gfnv2=&gfn2;
    tmp=clock();
    for(int j=0;j<N;j++) {
      for(int k=0;k<N;k++) {
	xa[0]=0.5;
	xa[1]=0.5;
	cr42.msolve(2,xa,vp,gfnv2);
      }
    }
    t5+=(clock()-tmp)/10000;
    cout << (clock()-tmp)/10000 << " " << x[0] << " " << x[1] << endl;
    t.test_rel(x[0],0.25,1.0e-6,"5a");
    t.test_rel(x[1],0.2,1.0e-6,"5b");

    cout << endl;
  }

  cout << t1 << endl;
  cout << t2 << endl;
  cout << t4 << endl;
  cout << t5 << endl;

  t.report();
  return 0;
}

