#include <o2scl/test_mgr.h>
#include <o2scl/jacobian.h>

using namespace std;
using namespace o2scl;

int tfun(size_t nv, const ovector_view &x, ovector_view &y, void *&vp) {
  y[0]=x[0]*x[0]+sin(x[1]);
  y[1]=pow(x[1],3.0)+tan(x[0]);
  return 0;
}

int main(void) {
  void *vp=0;

  exact_jacobian<void *,mm_funct_fptr<void *> > ej;
  simple_jacobian<void *,mm_funct_fptr<void *> > sj;

  test_mgr t;
  t.set_output_level(2);

  mm_funct_fptr<void *> mff(tfun);
  
  ej.set_function(mff);
  sj.set_function(mff);

  ovector x(2), y(2);
  omatrix jac(2,2), jex(2,2);

  x[0]=2.0;
  x[1]=3.0;

  tfun(2,x,y,vp);

  cout.setf(ios::scientific);
  cout.precision(6);

  jex[0][0]=2.0*x[0];
  jex[0][1]=cos(x[1]);
  jex[1][0]=1.0/cos(x[0])/cos(x[0]);
  jex[1][1]=3.0*x[1]*x[1];

  cout << jex[0][0] << " " << jex[0][1] << endl;
  cout << jex[1][0] << " " << jex[1][1] << endl;
  cout << endl;

  sj(2,x,y,jac,vp);
  cout << jac[0][0] << " " << jac[0][1] << endl;
  cout << jac[1][0] << " " << jac[1][1] << endl;
  cout << endl;

  t.test_rel(jac[0][0],jex[0][0],1.0e-3,"simple");
  t.test_rel(jac[0][1],jex[0][1],1.0e-3,"simple");
  t.test_rel(jac[1][0],jex[1][0],1.0e-3,"simple");
  t.test_rel(jac[1][1],jex[1][1],1.0e-3,"simple");
  cout << endl;
  
  ej(2,x,y,jac,vp);
  cout << jac[0][0] << " " << jac[0][1] << endl;
  cout << jac[1][0] << " " << jac[1][1] << endl;
  cout << endl;
  
  t.test_rel(jac[0][0],jex[0][0],1.0e-10,"exact");
  t.test_rel(jac[0][1],jex[0][1],1.0e-10,"exact");
  t.test_rel(jac[1][0],jex[1][0],1.0e-10,"exact");
  t.test_rel(jac[1][1],jex[1][1],1.0e-10,"exact");
  cout << endl;

  cout << 2.0*x[0] << " " << cos(x[1]) << endl;
  cout << 1.0/cos(x[0])/cos(x[0]) << " "
       << 3.0*x[1]*x[1] << endl;
  cout << endl;

  t.report();
  return 0;
}

